from __future__ import annotations

from dataclasses import dataclass

import evaluate
import numpy as np

from chateval.metrics.metric import Metric, MetricConfig


@dataclass
class RougeConfig(MetricConfig):
    name = "rouge"
    variety: str = "rouge1"

    def to_metric(self) -> Rouge:
        return Rouge(self)


class Rouge(Metric):
    def __init__(self, config: RougeConfig):
        self.config = config
        self.evaluator = evaluate.load(self.config.name)

    def compute(self, dataset: list[dict], predictions: list[str]) -> dict:

        results = self.evaluator.compute(
            predictions=predictions,
            references=[sample["references"] for sample in dataset],
            use_aggregator=False,
            rouge_types=[self.config.variety],
        )

        return {
            "value": np.mean(results[self.config.variety]),
            "sample_values": results[self.config.variety],
        }



from abc import ABC, abstractmethod
import bz2
import gzip
import lzma
import os
import pathlib
import shutil
import tarfile
from typing import Optional, Union
import warnings
import zipfile

from chateval import config
from chateval.utils.filelock import FileLock
from chateval.utils.logging import get_logger
from chateval.utils.py_util import hash_url_to_filename

logger = get_logger(__name__)


class ExtractManager:
    def __init__(self, cache_dir: Optional[str] = None):
        self.extract_dir = (
            os.path.join(cache_dir, config.EXTRACTED_SCENARIO_DIR)
            if cache_dir
            else config.EXTRACTED_SCENARIO_PATH
        )
        self.extractor = Extractor

    def _get_output_path(self, path: str) -> str:
        # Path where we extract compressed archives
        # We extract in the cache dir, and get the extracted path name by
        # hashing the original path"
        abs_path = os.path.abspath(path)
        return os.path.join(
            self.extract_dir, hash_url_to_filename(abs_path)
        )  # type: ignore

    def _do_extract(self, output_path: str, force_extract: bool):  # type: ignore
        return force_extract or (
            not os.path.isfile(output_path)
            and not (os.path.isdir(output_path) and os.listdir(output_path))
        )

    def extract(self, input_path: str, force_extract: bool = False) -> str:
        extractor_format = self.extractor.infer_extractor_format(input_path)
        if not extractor_format:
            return input_path
        output_path = self._get_output_path(input_path)
        if self._do_extract(output_path, force_extract):
            self.extractor.extract(input_path, output_path, extractor_format)
        return output_path


class BaseExtractor(ABC):
    @classmethod
    @abstractmethod
    def is_extractable(cls, path: Union["pathlib.Path", str], **kwargs) -> bool:
        ...

    @staticmethod
    @abstractmethod
    def extract(
        input_path: Union["pathlib.Path", str], output_path: Union["pathlib.Path", str]
    ) -> None:
        ...


class MagicNumberBaseExtractor(BaseExtractor, ABC):
    magic_number = b""

    @staticmethod
    def read_magic_number(path: Union["pathlib.Path", str], magic_number_length: int):
        with open(path, "rb") as f:
            return f.read(magic_number_length)

    @classmethod
    def is_extractable(  # type: ignore
        cls, path: Union["pathlib.Path", str], magic_number: bytes = b""
    ) -> bool:
        if not magic_number:
            try:
                magic_number = cls.read_magic_number(path, len(cls.magic_number))
            except OSError:
                return False
        return magic_number.startswith(cls.magic_number)


class TarExtractor(BaseExtractor):
    @classmethod
    def is_extractable(cls, path: Union["pathlib.Path", str], **kwargs) -> bool:
        return tarfile.is_tarfile(path)

    @staticmethod
    def safemembers(members):
        """
        Fix for CVE-2007-4559
        Desc:
            Directory traversal vulnerability in the (1) extract and (2) extractall
             functions in the tarfile module in Python allows user-assisted remote
              attackers to overwrite arbitrary files via a .. (dot dot)
            sequence in filenames in a TAR archive, a related issue to CVE-2001-1267.
        See: https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2007-4559
        From: https://stackoverflow.com/a/10077309
        """

        def resolved(path: str) -> str:
            return os.path.realpath(os.path.abspath(path))

        def badpath(path: str, base: str) -> bool:
            # joinpath will ignore base if path is absolute
            return not resolved(os.path.join(base, path)).startswith(base)

        def badlink(info, base: str) -> bool:
            # Links are interpreted relative to the directory containing the link
            tip = resolved(os.path.join(base, os.path.dirname(info.name)))
            return badpath(info.linkname, base=tip)

        base = resolved(".")

        for finfo in members:
            if badpath(finfo.name, base):
                logger.error(f"Extraction of {finfo.name} is blocked (illegal path)")
            elif finfo.issym() and badlink(finfo, base):
                logger.error(
                    f"Extraction of {finfo.name} is blocked: Hard"
                    f" link to {finfo.linkname}"
                )
            elif finfo.islnk() and badlink(finfo, base):
                logger.error(
                    f"Extraction of {finfo.name} is blocked:"
                    f" Symlink to {finfo.linkname}"
                )
            else:
                yield finfo

    @staticmethod
    def extract(
        input_path: Union["pathlib.Path", str], output_path: Union["pathlib.Path", str]
    ) -> None:
        os.makedirs(output_path, exist_ok=True)
        tar_file = tarfile.open(input_path)
        tar_file.extractall(output_path, members=TarExtractor.safemembers(tar_file))
        tar_file.close()


class GzipExtractor(MagicNumberBaseExtractor):
    magic_number = b"\x1F\x8B"

    @staticmethod
    def extract(
        input_path: Union["pathlib.Path", str], output_path: Union["pathlib.Path", str]
    ) -> None:
        with gzip.open(input_path, "rb") as gzip_file:
            with open(output_path, "wb") as extracted_file:
                shutil.copyfileobj(gzip_file, extracted_file)


class ZipExtractor(BaseExtractor):
    @classmethod
    def is_extractable(cls, path: Union["pathlib.Path", str], **kwargs) -> bool:
        return zipfile.is_zipfile(path)

    @staticmethod
    def extract(
        input_path: Union["pathlib.Path", str], output_path: Union["pathlib.Path", str]
    ) -> None:
        os.makedirs(output_path, exist_ok=True)
        with zipfile.ZipFile(input_path, "r") as zip_file:
            zip_file.extractall(output_path)
            zip_file.close()


class XzExtractor(MagicNumberBaseExtractor):
    magic_number = b"\xFD\x37\x7A\x58\x5A\x00"

    @staticmethod
    def extract(
        input_path: Union["pathlib.Path", str], output_path: Union["pathlib.Path", str]
    ) -> None:
        with lzma.open(input_path) as compressed_file:
            with open(output_path, "wb") as extracted_file:
                shutil.copyfileobj(compressed_file, extracted_file)


class RarExtractor(BaseExtractor):
    RAR_ID = b"Rar!\x1a\x07\x00"
    RAR5_ID = b"Rar!\x1a\x07\x01\x00"

    @classmethod
    def is_extractable(cls, path: Union["pathlib.Path", str], **kwargs) -> bool:
        """https://github.com/markokr/rarfile/blob/master/rarfile.py"""
        with open(path, "rb") as f:
            magic_number = f.read(len(cls.RAR5_ID))
        return magic_number == cls.RAR5_ID or magic_number.startswith(cls.RAR_ID)

    @staticmethod
    def extract(
        input_path: Union["pathlib.Path", str], output_path: Union["pathlib.Path", str]
    ) -> None:
        if not config.RARFILE_AVAILABLE:  # type: ignore
            raise ImportError("Please pip install rarfile")
        import rarfile

        os.makedirs(output_path, exist_ok=True)
        rf = rarfile.RarFile(input_path)
        rf.extractall(output_path)
        rf.close()


class ZstdExtractor(MagicNumberBaseExtractor):
    magic_number = b"\x28\xb5\x2F\xFD"

    @staticmethod
    def extract(
        input_path: Union["pathlib.Path", str], output_path: Union["pathlib.Path", str]
    ) -> None:
        if not config.ZSTANDARD_AVAILABLE:  # type: ignore
            raise ImportError("Please pip install zstandard")
        import zstandard as zstd

        dctx = zstd.ZstdDecompressor()
        with open(input_path, "rb") as ifh, open(output_path, "wb") as ofh:
            dctx.copy_stream(ifh, ofh)


class Bzip2Extractor(MagicNumberBaseExtractor):
    magic_number = b"\x42\x5A\x68"

    @staticmethod
    def extract(
        input_path: Union["pathlib.Path", str], output_path: Union["pathlib.Path", str]
    ) -> None:
        with bz2.open(input_path, "rb") as compressed_file:
            with open(output_path, "wb") as extracted_file:
                shutil.copyfileobj(compressed_file, extracted_file)


class SevenZipExtractor(MagicNumberBaseExtractor):
    magic_number = b"\x37\x7A\xBC\xAF\x27\x1C"

    @staticmethod
    def extract(
        input_path: Union["pathlib.Path", str], output_path: Union["pathlib.Path", str]
    ) -> None:
        if not config.PY7ZR_AVAILABLE:  # type: ignore
            raise ImportError("Please pip install py7zr")
        import py7zr

        os.makedirs(output_path, exist_ok=True)
        with py7zr.SevenZipFile(input_path, "r") as archive:
            archive.extractall(output_path)


class Lz4Extractor(MagicNumberBaseExtractor):
    magic_number = b"\x04\x22\x4D\x18"

    @staticmethod
    def extract(
        input_path: Union["pathlib.Path", str], output_path: Union["pathlib.Path", str]
    ) -> None:
        if not config.LZ4_AVAILABLE:  # type: ignore
            raise ImportError("Please pip install lz4")
        import lz4.frame

        with lz4.frame.open(input_path, "rb") as compressed_file:
            with open(output_path, "wb") as extracted_file:
                shutil.copyfileobj(compressed_file, extracted_file)


class Extractor:
    #  Put zip file to the last, b/c it is possible wrongly detected as zip
    #  (I guess it means: as tar or gzip)
    extractors = {
        "tar": TarExtractor,
        "gzip": GzipExtractor,
        "zip": ZipExtractor,
        "xz": XzExtractor,
        "rar": RarExtractor,
        "zstd": ZstdExtractor,
        "bz2": Bzip2Extractor,
        "7z": SevenZipExtractor,  # <Added version="2.4.0"/>
        "lz4": Lz4Extractor,  # <Added version="2.4.0"/>
    }

    @classmethod
    def _get_magic_number_max_length(cls):
        magic_number_max_length = 0
        for extractor in cls.extractors.values():
            if hasattr(extractor, "magic_number"):
                magic_number_length = len(extractor.magic_number)
                magic_number_max_length = (
                    magic_number_length
                    if magic_number_length > magic_number_max_length
                    else magic_number_max_length
                )
        return magic_number_max_length

    @staticmethod
    def _read_magic_number(path: Union["pathlib.Path", str], magic_number_length: int):
        try:
            return MagicNumberBaseExtractor.read_magic_number(
                path, magic_number_length=magic_number_length
            )
        except OSError:
            return b""

    @classmethod
    def is_extractable(
        cls, path: Union["pathlib.Path", str], return_extractor: bool = False
    ) -> bool:
        warnings.warn(
            "Method 'is_extractable' was deprecated in version 2.4.0 and"
            " will be removed in 3.0.0. "
            "Use 'infer_extractor_format' instead.",
            category=FutureWarning,
        )
        extractor_format = cls.infer_extractor_format(path)
        if extractor_format:
            return (
                True
                if not return_extractor
                else (True, cls.extractors[extractor_format])  # type: ignore
            )
        return False if not return_extractor else (False, None)  # type: ignore

    @classmethod
    def infer_extractor_format(
        cls, path: Union["pathlib.Path", str]
    ) -> str:  # <Added version="2.4.0"/>
        magic_number_max_length = cls._get_magic_number_max_length()
        magic_number = cls._read_magic_number(path, magic_number_max_length)
        for extractor_format, extractor in cls.extractors.items():
            if extractor.is_extractable(path, magic_number=magic_number):
                return extractor_format
        return ""

    @classmethod
    def extract(
        cls,
        input_path: Union["pathlib.Path", str],
        output_path: Union["pathlib.Path", str],
        extractor_format: Optional[str] = None,  # <Added version="2.4.0"/>
        extractor: Optional[BaseExtractor] = "deprecated",  # type: ignore
    ) -> None:
        # Prevent parallel extractions
        lock_path = str(input_path) + ".lock"
        with FileLock(lock_path):  # type: ignore
            shutil.rmtree(output_path, ignore_errors=True)
            os.makedirs(os.path.dirname(output_path), exist_ok=True)
            if extractor_format or extractor != "deprecated":
                if extractor != "deprecated" or not isinstance(
                    extractor_format, str
                ):  # passed as positional arg
                    warnings.warn(
                        "Parameter 'extractor' was deprecated in version 2.4.0 and"
                        " will be removed in 3.0.0. "
                        "Use 'extractor_format' instead.",
                        category=FutureWarning,
                    )
                    extractor = (
                        extractor
                        if extractor != "deprecated"
                        else extractor_format  # type: ignore
                    )
                else:
                    extractor = cls.extractors[extractor_format]  # type: ignore # noqa
                return extractor.extract(input_path, output_path)  # type: ignore
            else:
                warnings.warn(
                    "Parameter 'extractor_format' was made required in"
                    " version 2.4.0 and not passing it will raise an "
                    "exception in 3.0.0.",
                    category=FutureWarning,
                )
                for extractor in cls.extractors.values():  # type: ignore
                    if extractor.is_extractable(input_path):  # type: ignore
                        return extractor.extract(input_path, output_path)  # type: ignore # noqa
        return False  # type: ignore



from __future__ import annotations

from dataclasses import dataclass


@dataclass
class APIModel:
    content: str


@dataclass
class MetricModel:
    content: str


@dataclass
class PredictionModel:
    content: list[str]



import pathlib
import unittest

from chateval import load

current_path = pathlib.Path(__file__).resolve().parent


class MyTestCase(unittest.TestCase):
    @unittest.skip("this requires openai's api key")
    def test_scenario(self):

        scenario = load(f"{current_path}")
        metric_model = scenario.get_default_setting_config()["metric_model"]
        result = scenario.evaluate(metric_model, "metric")

        print(result)

        self.assertAlmostEqual(result["accuracy"]["value"], 0, 3)



from __future__ import annotations

import string


def format_necessary(template: str, **kwargs: dict[str, str]) -> str:
    """Format a template string with only necessary kwargs."""
    keys = [k[1] for k in string.Formatter().parse(template) if k[1]]
    assert all(k in kwargs for k in keys), f"Required: {keys}, got: {sorted(kwargs)}"
    cur_keys = {k: kwargs[k] for k in keys}
    return template.format(**cur_keys)



from contextlib import closing, contextmanager
import copy
from dataclasses import dataclass
from functools import partial
from hashlib import sha256
import json
import os
from pathlib import Path
import posixpath
import re
import shutil
import tempfile
import time
from typing import Optional, TypeVar, Union
import urllib
from urllib.parse import urljoin, urlparse

import requests

from chateval import config
from chateval.utils import logging
from chateval.utils.extract import ExtractManager
from chateval.utils.filelock import FileLock

logger = logging.get_logger(__name__)

T = TypeVar("T", str, Path)


def is_remote_url(url_or_filename: str) -> bool:
    parsed = urlparse(url_or_filename)
    return parsed.scheme in ("http", "https", "s3", "gs", "hdfs", "ftp")


def url_or_path_join(base_name: str, *pathnames: str) -> str:
    if is_remote_url(base_name):
        return posixpath.join(
            base_name,
            *(str(pathname).replace(os.sep, "/").lstrip("/") for pathname in pathnames),
        )
    else:
        return Path(base_name, *pathnames).as_posix()


def is_local_path(url_or_filename: str) -> bool:
    # On unix the scheme of a local path is empty (for both absolute and relative),
    # while on windows the scheme is the drive name (ex: "c") for absolute paths.
    # for details on the windows behavior, see https://bugs.python.org/issue42215
    return urlparse(url_or_filename).scheme == "" or os.path.ismount(
        urlparse(url_or_filename).scheme + ":/"
    )


def is_relative_path(url_or_filename: str) -> bool:
    return urlparse(url_or_filename).scheme == "" and not os.path.isabs(url_or_filename)


def relative_to_absolute_path(path: T) -> T:
    """Convert relative path to absolute path."""
    abs_path_str = os.path.abspath(os.path.expanduser(os.path.expandvars(str(path))))
    return Path(abs_path_str) if isinstance(path, Path) else abs_path_str


def ep_github_url(
    path: str, name: str, scenario=True, revision: Optional[str] = None
) -> str:

    revision = revision or os.getenv("EP_SCRIPTS_VERSION", config.SCRIPTS_VERSION)
    if scenario:
        return config.HUB_SCENARIO_URL.format(revision=revision, path=path, name=name)
    return False  # type: ignore


def cached_path(
    url_or_filename,
    download_config=None,
    **download_kwargs,
) -> str:
    """
    Given something that might be a URL (or might be a local path),
    determine which. If it's a URL, download the file and cache it, and
    return the path to the cached file. If it's already a local path,
    make sure the file exists and then return the path.
    Return:
        Local path (string)
    Raises:
        FileNotFoundError: in case of non-recoverable file
            (non-existent or no cache on disk)
        ConnectionError: in case of unreachable url
            and no cache on disk
        ValueError: if it couldn't parse the url or filename correctly
        requests.exceptions.ConnectionError: in case of internet connection issue
    """
    if download_config is None:
        download_config = DownloadConfig(**download_kwargs)

    cache_dir = download_config.cache_dir or config.DOWNLOADED_SCENARIO_PATH
    if isinstance(cache_dir, Path):
        cache_dir = str(cache_dir)
    if isinstance(url_or_filename, Path):
        url_or_filename = str(url_or_filename)

    if is_remote_url(url_or_filename):
        # URL, so get it from the cache (downloading if necessary)
        output_path = get_from_cache(
            url_or_filename,
            cache_dir=cache_dir,
            force_download=download_config.force_download,
            proxies=download_config.proxies,
            resume_download=download_config.resume_download,
            user_agent=download_config.user_agent,
            local_files_only=download_config.local_files_only,
            use_etag=download_config.use_etag,
            max_retries=download_config.max_retries,
            use_auth_token=download_config.use_auth_token,
            ignore_url_params=download_config.ignore_url_params,
            download_desc=download_config.download_desc,
        )
    elif os.path.exists(url_or_filename):
        # File, and it exists.
        output_path = url_or_filename
    elif is_local_path(url_or_filename):
        # File, but it doesn't exist.
        raise FileNotFoundError(f"Local file {url_or_filename} doesn't exist")
    else:
        # Something unknown
        raise ValueError(
            f"unable to parse {url_or_filename} as a URL or as a local path"
        )

    if output_path is None:
        return output_path

    if download_config.extract_compressed_file:
        output_path = ExtractManager(cache_dir=download_config.cache_dir).extract(
            output_path, force_extract=download_config.force_extract
        )

    return output_path


@dataclass
class DownloadConfig:
    """Configuration for our cached path manager.
    Attributes:
        cache_dir (:obj:`str` or :obj:`Path`, optional): Specify a cache directory
         to save the file to (overwrite the
            default cache dir).
        force_download (:obj:`bool`, default ``False``): If True, re-dowload the
         file even if it's already cached in
            the cache dir.
        resume_download (:obj:`bool`, default ``False``): If True, resume the download
         if incompletly recieved file is
            found.
        proxies (:obj:`dict`, optional):
        user_agent (:obj:`str`, optional): Optional string or dict that will be
         appended to the user-agent on remote
            requests.
        extract_compressed_file (:obj:`bool`, default ``False``): If True and the
         path point to a zip or tar file,
            extract the compressed file in a folder along the archive.
        force_extract (:obj:`bool`, default ``False``): If True when
         extract_compressed_file is True and the archive
            was already extracted, re-extract the archive and override the folder
             where it was extracted.
        delete_extracted (:obj:`bool`, default ``False``): Whether to delete
         (or keep) the extracted files.
        use_etag (:obj:`bool`, default ``True``): Whether to use the ETag HTTP
         response header to validate the cached files.
        num_proc (:obj:`int`, optional): The number of processes to launch to
         download the files in parallel.
        max_retries (:obj:`int`, default ``1``): The number of times to retry
        an HTTP request if it fails.
        use_auth_token (:obj:`str` or :obj:`bool`, optional): Optional string
        or boolean to use as Bearer token
            for remote files on the Datasets Hub. If True, will get token
             from ~/.huggingface.
        ignore_url_params (:obj:`bool`, default ``False``): Whether to strip all
         query parameters and #fragments from
            the download URL before using it for caching the file.
        download_desc (:obj:`str`, optional): A description to be displayed
         alongside with the progress bar while downloading the files.
    """

    cache_dir: Optional[Union[str, Path]] = None
    force_download: bool = False
    resume_download: bool = False
    local_files_only: bool = False
    proxies: Optional[dict] = None
    user_agent: Optional[str] = None
    extract_compressed_file: bool = False
    force_extract: bool = False
    delete_extracted: bool = False
    use_etag: bool = True
    num_proc: Optional[int] = None
    max_retries: int = 1
    use_auth_token: Optional[Union[str, bool]] = None
    ignore_url_params: bool = False
    download_desc: Optional[str] = None

    def copy(self) -> "DownloadConfig":
        return self.__class__(**{k: copy.deepcopy(v) for k, v in self.__dict__.items()})


def ep_hub_url(path: str, name: str, revision: Optional[str] = None) -> str:
    revision = revision or config.HUB_DEFAULT_VERSION
    return config.HUB_SCENARIO_URL.format(path=path, name=name, revision=revision)


def hash_url_to_filename(url, etag=None):
    """
    Convert `url` into a hashed filename in a repeatable way.
    If `etag` is specified, append its hash to the url's, delimited
    by a period.
    If the url ends with .h5 (Keras HDF5 weights) adds '.h5' to the name
    so that TF 2.0 can identify it as a HDF5 file
    (see https://github.com/tensorflow/tensorflow/blob/00fad90125b18b80fe054d
    e1055770cfb8fe4ba3/tensorflow/python/keras/engine/network.py#L1380)
    """
    url_bytes = url.encode("utf-8")
    url_hash = sha256(url_bytes)
    filename = url_hash.hexdigest()

    if etag:
        etag_bytes = etag.encode("utf-8")
        etag_hash = sha256(etag_bytes)
        filename += "." + etag_hash.hexdigest()

    if url.endswith(".py"):
        filename += ".py"

    return filename


class OfflineModeIsEnabled(ConnectionError):
    pass


def _raise_if_offline_mode_is_enabled(msg: Optional[str] = None):
    """Raise an OfflineModeIsEnabled error (subclass of ConnectionError)
    if EP_EVALUATE_OFFLINE is True."""
    if config.EP_SCENARIO_OFFLINE:
        raise OfflineModeIsEnabled(
            "Offline mode is enabled."
            if msg is None
            else "Offline mode is enabled. " + str(msg)
        )


def ftp_head(url, timeout=10.0):
    _raise_if_offline_mode_is_enabled(f"Tried to reach {url}")
    try:
        with closing(urllib.request.urlopen(url, timeout=timeout)) as r:
            r.read(1)
    except Exception:
        return False
    return True


def get_authentication_headers_for_url(
    url: str, use_auth_token: Optional[Union[str, bool]] = None
) -> dict:
    """Handle the HF authentication"""
    headers = {}  # type: ignore
    return headers


def http_get(
    url,
    temp_file,
    proxies=None,
    resume_size=0,
    headers=None,
    cookies=None,
    timeout=100.0,
    max_retries=0,
    desc=None,
):
    headers = copy.deepcopy(headers) or {}
    headers["user-agent"] = "user-agent"  # TODO check
    if resume_size > 0:
        headers["Range"] = f"bytes={resume_size:d}-"
    response = _request_with_retry(
        method="GET",
        url=url,
        stream=True,
        proxies=proxies,
        headers=headers,
        cookies=cookies,
        max_retries=max_retries,
        timeout=timeout,
    )
    if response.status_code == 416:  # Range not satisfiable
        return
    content_length = response.headers.get("Content-Length")
    total = resume_size + int(content_length) if content_length is not None else None
    with logging.tqdm(
        unit="B",
        unit_scale=True,
        total=total,
        initial=resume_size,
        desc=desc or "Downloading",
        disable=not logging.is_progress_bar_enabled(),
    ) as progress:
        for chunk in response.iter_content(chunk_size=1024):
            progress.update(len(chunk))
            temp_file.write(chunk)


def http_head(
    url,
    proxies=None,
    headers=None,
    cookies=None,
    allow_redirects=True,
    timeout=10.0,
    max_retries=0,
) -> requests.Response:
    headers = copy.deepcopy(headers) or {}
    headers["user-agent"] = "user-agent"  # TODO: change
    response = _request_with_retry(
        method="HEAD",
        url=url,
        proxies=proxies,
        headers=headers,
        cookies=cookies,
        allow_redirects=allow_redirects,
        timeout=timeout,
        max_retries=max_retries,
    )
    return response


def _request_with_retry(
    method: str,
    url: str,
    max_retries: int = 0,
    base_wait_time: float = 0.5,
    max_wait_time: float = 2,
    timeout: float = 10.0,
    **params,
) -> requests.Response:
    """Wrapper around requests to retry in case it fails with a ConnectTimeout,
     with exponential backoff.
    Note that if the environment variable EP_EVALUATE_OFFLINE is set to 1,
     then a OfflineModeIsEnabled error is raised.
    Args:
        method (str): HTTP method, such as 'GET' or 'HEAD'.
        url (str): The URL of the resource to fetch.
        max_retries (int): Maximum number of retries, defaults to 0 (no retries).
        base_wait_time (float): Duration (in seconds) to wait before retrying the
         first time. Wait time between
            retries then grows exponentially, capped by max_wait_time.
        max_wait_time (float): Maximum amount of time between two retries, in seconds.
        **params: Params to pass to :obj:`requests.request`.
    """
    _raise_if_offline_mode_is_enabled(f"Tried to reach {url}")
    tries, success = 0, False
    while not success:
        tries += 1
        try:
            response = requests.request(
                method=method.upper(), url=url, timeout=timeout, **params
            )
            success = True
        except (
            requests.exceptions.ConnectTimeout,
            requests.exceptions.ConnectionError,
        ) as err:
            if tries > max_retries:
                raise err
            else:
                logger.info(
                    f"{method} request to {url} timed out,"
                    f" retrying... [{tries/max_retries}]"
                )
                sleep_time = min(
                    max_wait_time, base_wait_time * 2 ** (tries - 1)
                )  # Exponential backoff
                time.sleep(sleep_time)
    return response


def ftp_get(url, temp_file, timeout=10.0):
    _raise_if_offline_mode_is_enabled(f"Tried to reach {url}")
    try:
        logger.info(f"Getting through FTP {url} into {temp_file.name}")
        with closing(urllib.request.urlopen(url, timeout=timeout)) as r:
            shutil.copyfileobj(r, temp_file)
    except urllib.error.URLError as e:
        raise ConnectionError(e) from None


def get_from_cache(
    url,
    cache_dir=None,
    force_download=False,
    proxies=None,
    etag_timeout=100,
    resume_download=False,
    user_agent=None,
    local_files_only=False,
    use_etag=True,
    max_retries=0,
    use_auth_token=None,
    ignore_url_params=False,
    download_desc=None,
) -> str:
    """
    Given a URL, look for the corresponding file in the local cache.
    If it's not there, download it. Then return the path to the cached file.
    Return:
        Local path (string)
    Raises:
        FileNotFoundError: in case of non-recoverable file
            (non-existent or no cache on disk)
        ConnectionError: in case of unreachable url
            and no cache on disk
    """
    if cache_dir is None:
        cache_dir = config.EP_SCENARIO_CACHE
    if isinstance(cache_dir, Path):
        cache_dir = str(cache_dir)

    os.makedirs(cache_dir, exist_ok=True)

    if ignore_url_params:
        # strip all query parameters and #fragments from the URL
        cached_url = urljoin(url, urlparse(url).path)
    else:
        cached_url = url  # additional parameters may be added to the given URL

    connected = False
    response = None
    cookies = None
    etag = None
    head_error = None

    # Try a first time to file the file on the local file system without eTag (None)
    # if we don't ask for 'force_download' then we spare a request
    filename = hash_url_to_filename(cached_url, etag=None)
    cache_path = os.path.join(cache_dir, filename)

    if os.path.exists(cache_path) and not force_download and not use_etag:
        return cache_path

    # Prepare headers for authentication
    headers = get_authentication_headers_for_url(url, use_auth_token=use_auth_token)
    if user_agent is not None:
        headers["user-agent"] = user_agent

    # We don't have the file locally or we need an eTag
    if not local_files_only:
        if url.startswith("ftp://"):
            connected = ftp_head(url)
        try:
            response = http_head(
                url,
                allow_redirects=True,
                proxies=proxies,
                timeout=etag_timeout,
                max_retries=max_retries,
                headers=headers,
            )
            if response.status_code == 200:  # ok
                etag = response.headers.get("ETag") if use_etag else None
                for k, v in response.cookies.items():
                    # In some edge cases, we need to get a confirmation token
                    if k.startswith("download_warning") and "drive.google.com" in url:
                        url += "&confirm=" + v
                        cookies = response.cookies
                connected = True
                # Fix Google Drive URL to avoid Virus scan warning
                if "drive.google.com" in url and "confirm=" not in url:
                    url += "&confirm=t"
            # In some edge cases, head request returns 400 but the
            # connection is actually ok
            elif (
                (
                    response.status_code == 400
                    and "firebasestorage.googleapis.com" in url
                )
                or (response.status_code == 405 and "drive.google.com" in url)
                or (
                    response.status_code == 403
                    and (
                        re.match(
                            r"^https?://github.com/.*?/.*?/releases/download/.*?/.*?$",
                            url,
                        )
                        or re.match(
                            r"^https://.*?s3.*?amazonaws.com/.*?$", response.url
                        )
                    )
                )
                or (response.status_code == 403 and "ndownloader.figstatic.com" in url)
            ):
                connected = True
                logger.info(f"Couldn't get ETag version for url {url}")
            elif (
                response.status_code == 401
                and config.EP_ENDPOINT in url
                and use_auth_token is None
            ):
                raise ConnectionError(
                    f"Unauthorized for URL {url}. Please use the parameter"
                    f" ``use_auth_token=True`` after logging in with"
                    f" ``huggingface-cli login``"
                )
        except (OSError, requests.exceptions.Timeout) as e:
            # not connected
            head_error = e
            pass

    # connected == False = we don't have a connection, or url doesn't exist,
    # or is otherwise inaccessible.
    # try to get the last downloaded one
    if not connected:
        if os.path.exists(cache_path) and not force_download:
            return cache_path
        if local_files_only:
            raise FileNotFoundError(
                f"Cannot find the requested files in the cached path at {cache_path}"
                f" and outgoing traffic has been"
                " disabled. To enable file online look-ups, set 'local_files_only'"
                " to False."
            )
        elif response is not None and response.status_code == 404:
            raise FileNotFoundError(f"Couldn't find file at {url}")
        _raise_if_offline_mode_is_enabled(f"Tried to reach {url}")
        if head_error is not None:
            raise ConnectionError(f"Couldn't reach {url} ({repr(head_error)})")
        elif response is not None:
            raise ConnectionError(
                f"Couldn't reach {url} (error {response.status_code})"
            )
        else:
            raise ConnectionError(f"Couldn't reach {url}")

    # Try a second time
    filename = hash_url_to_filename(cached_url, etag)
    cache_path = os.path.join(cache_dir, filename)

    if os.path.exists(cache_path) and not force_download:
        return cache_path

    # From now on, connected is True.
    # Prevent parallel downloads of the same file with a lock.
    lock_path = cache_path + ".lock"
    with FileLock(lock_path):  # type: ignore

        if resume_download:
            incomplete_path = cache_path + ".incomplete"

            @contextmanager
            def _resumable_file_manager():
                with open(incomplete_path, "a+b") as f:
                    yield f

            temp_file_manager = _resumable_file_manager
            if os.path.exists(incomplete_path):
                resume_size = os.stat(incomplete_path).st_size
            else:
                resume_size = 0
        else:
            temp_file_manager = partial(
                tempfile.NamedTemporaryFile, dir=cache_dir, delete=False  # type: ignore
            )
            resume_size = 0

        # Download to temporary file, then copy to cache dir once finished.
        # Otherwise you get corrupt cache entries if the download gets interrupted.
        with temp_file_manager() as temp_file:
            logger.info(
                f"{url} not found in cache or force_download set to True,"
                f" downloading to {temp_file.name}"
            )

            # GET file object
            if url.startswith("ftp://"):
                ftp_get(url, temp_file)
            else:
                http_get(
                    url,
                    temp_file,
                    proxies=proxies,
                    resume_size=resume_size,
                    headers=headers,
                    cookies=cookies,
                    max_retries=max_retries,
                    desc=download_desc,
                )

        logger.info(f"storing {url} in cache at {cache_path}")
        shutil.move(temp_file.name, cache_path)

        logger.info(f"creating metadata file for {cache_path}")
        meta = {"url": url, "etag": etag}
        meta_path = cache_path + ".json"
        with open(meta_path, "w", encoding="utf-8") as meta_file:
            json.dump(meta, meta_file)

    return cache_path



import unittest

from chateval.metrics import get_metric
from chateval.metrics.rouge import Rouge, RougeConfig


class MyTestCase(unittest.TestCase):
    def test_ROUGE(self):

        dataset = [{"references": ["hello world"]}, {"references": ["hello world"]}]
        predictions = ["hello world", "hello world"]

        rouge = Rouge(RougeConfig(name="rouge", variety="rouge1"))

        result = rouge.compute(dataset, predictions)

        self.assertEqual(result["value"], 1.0)
        self.assertEqual(result["sample_values"], [1.0, 1.0])

    def test_get_metric(self):
        metric = get_metric("rouge1")
        self.assertIsInstance(metric, Rouge)

    def test_count(self):
        dataset = [{"references": ["hello world"]}, {"references": ["hello world"]}]
        predictions = ["hello world", "hello world"]

        metric = get_metric("count")
        result = metric.compute(dataset, predictions)

        self.assertEqual(result["value"], 2.0)
        self.assertEqual(result["sample_values"], [2.0, 2.0])

    def test_accuracy(self):
        dataset = [{"references": ["Yes"]}, {"references": ["Yes"]}]
        predictions = ["Yes", "No"]

        metric = get_metric("accuracy")
        result = metric.compute(dataset, predictions)

        print(result)

        self.assertEqual(result["value"], 0.5)
        self.assertEqual(result["sample_values"], [1, 0])


if __name__ == "__main__":
    unittest.main()



# Part of code is from Copyright 2020 The HuggingFace Datasets Authors and the
# TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, scenario
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import annotations

from dataclasses import dataclass
import filecmp
import importlib
import inspect
import json
import os
from pathlib import Path
import re
import shutil
import sys
import time
from typing import Any, Optional, Union
from urllib.parse import urlparse

from chateval import config
from chateval.chateval import ChatEval
from chateval.utils.download import DownloadMode
from chateval.utils.file_utils import (
    cached_path,
    DownloadConfig,
    ep_github_url,
    is_relative_path,
    relative_to_absolute_path,
    url_or_path_join,
)
from chateval.utils.filelock import FileLock
from chateval.utils.logging import get_logger
from chateval.utils.py_util import _hash_python_lines
from chateval.utils.version import Version

logger = get_logger(__name__)


def init_dynamic_modules(
    name: str = config.MODULE_NAME_FOR_DYNAMIC_MODULES,
    ep_modules_cache: Optional[Union[Path, str]] = None,
):
    """
    Create a module with name `name` in which you can add dynamic modules
    such as metrics or datasets. The module can be imported using its name.
    The module is created in the EP_MODULE_CACHE directory by default
     (~/.cache/expressai/modules) but it can
    be overriden by specifying a path to another directory in `ep_modules_cache`.
    """

    ep_modules_cache = init_ep_modules(ep_modules_cache)
    dynamic_modules_path = os.path.join(ep_modules_cache, name)
    os.makedirs(dynamic_modules_path, exist_ok=True)
    if not os.path.exists(os.path.join(dynamic_modules_path, "__init__.py")):
        with open(os.path.join(dynamic_modules_path, "__init__.py"), "w"):
            pass
    return dynamic_modules_path


def init_ep_modules(ep_modules_cache: Optional[Union[Path, str]] = None) -> str:
    """
    Add ep_modules_cache to the python path.
    By default ep_modules_cache='~/.cache/expressai/modules'.
    It can also be set with the environment variable ep_MODULES_CACHE.
    This is used to add modules such as `datasets_modules`
    """

    ep_modules_cache = (
        ep_modules_cache if ep_modules_cache is not None else config.EP_MODULES_CACHE
    )
    ep_modules_cache = str(ep_modules_cache)
    if ep_modules_cache not in sys.path:
        sys.path.append(ep_modules_cache)

        os.makedirs(ep_modules_cache, exist_ok=True)
        if not os.path.exists(os.path.join(ep_modules_cache, "__init__.py")):
            with open(os.path.join(ep_modules_cache, "__init__.py"), "w"):
                pass

    return ep_modules_cache


def import_main_class(
    module_path: str, target_class: type[Any], include_self=False
) -> type[Any]:
    """Import a module at module_path and return its main class:
    module_path|str: for example, chateval.module_test
    target_class:
    """
    module = importlib.import_module(module_path)
    main_cls_type = target_class

    # Find the main class in our imported module
    module_main_cls = None
    for name, obj in module.__dict__.items():
        # We only want to get the inheritance class of the `main_cls_type`
        if obj == main_cls_type and not include_self:
            continue
        if isinstance(obj, type) and issubclass(obj, main_cls_type):
            if inspect.isabstract(obj):
                continue
            module_main_cls = obj
            break

    return module_main_cls  # type: ignore


def files_to_hash(file_paths: list[str]) -> str:
    """
    Convert a list of scripts or text files provided in file_paths into a hashed
     filename in a repeatable way.
    """
    # List all python files in directories if directories are supplied
    # as part of external imports
    to_use_files: list[Union[Path, str]] = []
    for file_path in file_paths:
        if os.path.isdir(file_path):
            to_use_files.extend(list(Path(file_path).rglob("*.[pP][yY]")))
        else:
            to_use_files.append(file_path)

    # Get the code from all these files
    lines = []
    for file_path in to_use_files:  # type: ignore
        with open(file_path, encoding="utf-8") as f:
            lines.extend(f.readlines())
    return _hash_python_lines(lines)


def convert_github_url(url_path: str) -> tuple[str, Optional[str]]:
    """Convert a link to a file on a github repo in a link to the raw github object.
    Input:
        url_path: https://github.com/ExpressAI/ChatEval
    Output:
        https://github.com/ExpressAI/ChatEval/archive/main.zip
        ChatEval-main
    """
    parsed = urlparse(url_path)
    sub_directory = None
    if parsed.scheme in ("http", "https", "s3") and parsed.netloc == "github.com":
        if "blob" in url_path:
            if not url_path.endswith(".py"):
                raise ValueError(
                    f"External import from github at {url_path} should"
                    f" point to a file ending with '.py'"
                )
            url_path = url_path.replace("blob", "raw")  # Point to the raw file
        else:
            # Parse github url to point to zip
            github_path = parsed.path[1:]
            repo_info, branch = (
                github_path.split("/tree/")
                if "/tree/" in github_path
                else (github_path, "main")
            )
            # TODO: check (master or main)
            repo_owner, repo_name = repo_info.split("/")
            url_path = (
                f"https://github.com/{repo_owner}/{repo_name}/" f"archive/{branch}.zip"
            )
            sub_directory = f"{repo_name}-{branch}"
    return url_path, sub_directory


def get_imports(file_path: str) -> tuple[str, str, str, str]:
    """Find whether we should import or clone additional files for a
     given processing script.
        And list the import.
    We allow:
    - library dependencies,
    - local dependencies and
    - external dependencies whose url is specified with a comment starting from
     "# From:' followed by the raw url to a file, an archive or a github repository.
        external dependencies will be downloaded (and extracted if needed in
         the dataset folder).
        We also add an `__init__.py` to each sub-folder of a downloaded folder
         so the user can import from them in the script.
    Note that only direct import in the dataset processing script will be handled
    We don't recursively explore the additional import to download further files.
    Example::
        import tensorflow
        import .c4_utils
        import .clicr.dataset-code.build_json_dataset  # From:
        https://github.com/ExpressAI/DataLab/blob/main/datalabs/
    """
    lines = []
    with open(file_path, encoding="utf-8") as f:
        lines.extend(f.readlines())

    logger.debug(f"Checking {file_path} for additional imports.")
    imports: list[tuple[str, str, str, Optional[str]]] = []
    is_in_docstring = False
    for line in lines:
        docstr_start_match = re.findall(r'[\s\S]*?"""[\s\S]*?', line)

        if len(docstr_start_match) == 1:
            # flip True <=> False only if doctstring
            # starts at line without finishing
            is_in_docstring = not is_in_docstring

        if is_in_docstring:
            # import statements in doctstrings should
            # not be added as required dependencies
            continue

        match = re.match(
            r"^import\s+(\.?)([^\s\.]+)[^#\r\n]*(?:#\s+From:\s+)?([^\r\n]*)",
            line,
            flags=re.MULTILINE,
        )
        if match is None:
            match = re.match(
                r"^from\s+(\.?)([^\s\.]+)(?:[^\s]*)\s+import\s+[^#\r\n]*("
                r"?:#\s+From:\s+)?([^\r\n]*)",
                # noqa
                line,
                flags=re.MULTILINE,
            )
            if match is None:
                continue
        if match.group(1):
            # The import starts with a '.', we will download the relevant file
            if any(imp[1] == match.group(2) for imp in imports):
                # We already have this import
                continue
            if match.group(3):
                # The import has a comment with 'From:', we'll retrieve it
                # from the given url
                url_path = match.group(3)
                url_path, sub_directory = convert_github_url(url_path)
                imports.append(("external", match.group(2), url_path, sub_directory))
            elif match.group(2):
                # The import should be at the same place as the file
                imports.append(("internal", match.group(2), match.group(2), None))
        else:
            if match.group(3):
                # The import has a comment with `From: git+https:...`, asks
                # user to pip install from git.
                url_path = match.group(3)
                imports.append(("library", match.group(2), url_path, None))
            else:
                imports.append(("library", match.group(2), match.group(2), None))

    return imports  # type: ignore


def _download_additional_modules(
    name: str,
    base_path: str,
    imports: tuple[str, str, str, str],
    download_config: Optional[DownloadConfig],
) -> list[tuple[str, str]]:
    """
    Download additional module for a module <name>.py at URL (or local path)
     <base_path>/<name>.py The imports must have been parsed first using
      ``get_imports``.
    If some modules need to be installed with pip, an error is raised showing
     how to install them. This function return the list of downloaded modules as
     tuples (import_name, module_file_path).
    The downloaded modules can then be moved into an importable directory
     with ``_copy_script_and_other_resources_in_importable_dir``.
    """
    local_imports = []
    library_imports = []
    download_config = download_config.copy()  # type: ignore
    if download_config.download_desc is None:
        download_config.download_desc = "Downloading extra modules"
    for import_type, import_name, import_path, sub_directory in imports:  # type: ignore
        if import_type == "library":  # type: ignore
            library_imports.append((import_name, import_path))  # type: ignore
            continue

        if import_name == name:  # type: ignore
            raise ValueError(
                f"Error in the {name} script, importing relative "
                f"{import_name}"  # type: ignore
                f" module "
                f"but {import_name} is the name of the script. "  # type: ignore
                f"Please change relative import {import_name} "  # type: ignore
                f"to another name"
                f" and add a '# From: URL_OR_PATH' "
                f"comment pointing to the original relative import file path."
            )
        if import_type == "internal":  # type: ignore
            url_or_filename = url_or_path_join(
                base_path, import_path + ".py"  # type: ignore
            )  # type: ignore
        elif import_type == "external":  # type: ignore
            url_or_filename = import_path  # type: ignore
        else:
            raise ValueError("Wrong import_type")

        local_import_path = cached_path(
            url_or_filename,
            download_config=download_config,
        )
        if sub_directory is not None:  # type: ignore
            local_import_path = os.path.join(
                local_import_path, sub_directory  # type: ignore
            )  # type: ignore
        local_imports.append((import_name, local_import_path))  # type: ignore

    # Check library imports
    needs_to_be_installed = set()
    for library_import_name, library_import_path in library_imports:
        try:
            lib = importlib.import_module(library_import_name)  # noqa F841
        except ImportError:
            needs_to_be_installed.add((library_import_name, library_import_path))
    if needs_to_be_installed:
        raise ImportError(
            f"To be able to use {name}, you need to install the following"
            f" dependencies"
            f"{[lib_name for lib_name, lib_path in needs_to_be_installed]}"
            f" using 'pip install "
            f"{' '.join([lib_path for lib_name, lib_path in needs_to_be_installed])}'"
            f" for instance'"
        )
    return local_imports


def _copy_script_and_other_resources_in_importable_dir(
    name: str,
    importable_directory_path: str,
    subdirectory_name: str,
    original_local_path: str,
    local_imports: list[tuple[str, str]],
    additional_files: list[tuple[str, str]],
    download_mode: Optional[DownloadMode],
) -> str:
    """Copy a script and its required imports to an importable directory, one example:
    (1) subdirectory_name: a379d530807a50a4137ed4dda28a31f5
    24f8f457fcac901cfcb5e2c07c91c128
    (2) original_local_path: load_dataset(original_local_path="./datasets/mr")
    (3) importable_directory_path：
    /usr2/home/stefan/.cache/expressai/modules/datasets_modules/datasets/adv_mtl
    finally:
     /usr2/home/stefan/.cache/expressai/modules/datasets_modules/datasets/adv_mtl/
    a379d530807a50a4137ed4dda28a31f524f8f457fcac901cfcb5e2c07c91c128/adv_mtl.py
    Args:
        name (str): name of the resource to load
        importable_directory_path (str): path to the loadable folder in the
         dynamic modules directory
        subdirectory_name (str): name of the subdirectory in
        importable_directory_path in which to place the script
        original_local_path (str): local path to the resource script
        local_imports (List[Tuple[str, str]]): list of (destination_filename,
         import_file_to_copy)
        additional_files (List[Tuple[str, str]]): list of (destination_filename,
         additional_file_to_copy)
        download_mode (Optional[DownloadMode]): download mode
    Return:
        importable_local_file: path to an importable module with
         importlib.import_module
    """

    # Define a directory with a unique name in our dataset or metric folder
    # path is: ./datasets|metrics/dataset|metric_name/hash_from_code/script.py
    # we use a hash as subdirectory_name to be able to have multiple versions of
    # a dataset/metric processing file together
    importable_subdirectory = os.path.join(importable_directory_path, subdirectory_name)
    importable_local_file = os.path.join(importable_subdirectory, name + ".py")

    # Prevent parallel disk operations
    lock_path = importable_directory_path + ".lock"
    with FileLock(lock_path):  # type: ignore
        # Create main dataset/metrics folder if needed
        if download_mode == DownloadMode.FORCE_REDOWNLOAD and os.path.exists(
            importable_directory_path
        ):
            shutil.rmtree(importable_directory_path)
        os.makedirs(importable_directory_path, exist_ok=True)

        # add an __init__ file to the main dataset folder if needed
        init_file_path = os.path.join(importable_directory_path, "__init__.py")
        if not os.path.exists(init_file_path):
            with open(init_file_path, "w"):
                pass

        # Create hash dataset folder if needed
        os.makedirs(importable_subdirectory, exist_ok=True)
        # add an __init__ file to the hash dataset folder if needed
        init_file_path = os.path.join(importable_subdirectory, "__init__.py")
        if not os.path.exists(init_file_path):
            with open(init_file_path, "w"):
                pass

        # Copy dataset.py file in hash folder if needed
        if not os.path.exists(importable_local_file):
            shutil.copyfile(original_local_path, importable_local_file)

        # Record metadata associating original dataset path with local unique folder
        meta_path = importable_local_file.split(".py")[0] + ".json"
        if not os.path.exists(meta_path):
            meta = {
                "original file path": original_local_path,
                "local file path": importable_local_file,
            }
            # the filename is *.py in our case, so better rename to filenam.json
            # instead of filename.py.json
            with open(meta_path, "w", encoding="utf-8") as meta_file:
                json.dump(meta, meta_file)

        # Copy all the additional imports
        for import_name, import_path in local_imports:
            if os.path.isfile(import_path):
                full_path_local_import = os.path.join(
                    importable_subdirectory, import_name + ".py"
                )
                if not os.path.exists(full_path_local_import):
                    shutil.copyfile(import_path, full_path_local_import)
            elif os.path.isdir(import_path):
                full_path_local_import = os.path.join(
                    importable_subdirectory, import_name
                )
                if not os.path.exists(full_path_local_import):
                    shutil.copytree(import_path, full_path_local_import)
            else:
                raise OSError(f"Error with local import at {import_path}")

        # original_local_directory = os.path.dirname(original_local_path)
        # Copy additional files like dataset infos file if needed
        # additional_files = [("data", f"{original_local_directory}/data")]

        for file_name, original_path in additional_files:

            # if the file is a directory, copy the whole directory
            if os.path.isdir(original_path):
                destination_additional_path = os.path.join(
                    importable_subdirectory, file_name
                )
                if not os.path.exists(destination_additional_path):
                    shutil.copytree(original_path, destination_additional_path)
            else:
                destination_additional_path = os.path.join(
                    importable_subdirectory, file_name
                )
                if not os.path.exists(destination_additional_path) or not filecmp.cmp(
                    original_path, destination_additional_path
                ):
                    shutil.copyfile(original_path, destination_additional_path)
        return importable_local_file


def _create_importable_file(
    local_path: str,
    local_imports: list[tuple[str, str]],
    additional_files: list[tuple[str, str]],
    dynamic_modules_path: str,
    module_namespace: str,
    name: str,
    download_mode: DownloadMode,
) -> tuple[str, str]:
    """
    Args:
        local_path:
        local_imports:
        additional_files:
        dynamic_modules_path: /usr2/home/pliu3/.cache/expressai/
        modules/datasets_modules
        module_namespace: datasets
        name: adv_mtl
        download_mode:
    Returns:
    """
    importable_directory_path = os.path.join(
        dynamic_modules_path, module_namespace, name.replace("/", "--")
    )
    Path(importable_directory_path).mkdir(parents=True, exist_ok=True)
    (Path(importable_directory_path).parent / "__init__.py").touch(exist_ok=True)
    hash = files_to_hash([local_path] + [loc[1] for loc in local_imports])
    importable_local_file = _copy_script_and_other_resources_in_importable_dir(
        name=name.split("/")[-1],
        importable_directory_path=importable_directory_path,
        subdirectory_name=hash,
        original_local_path=local_path,
        local_imports=local_imports,
        additional_files=additional_files,
        download_mode=download_mode,
    )
    logger.debug(f"Created importable dataset file at {importable_local_file}")
    module_path = ".".join(
        [
            os.path.basename(dynamic_modules_path),
            module_namespace,
            name.replace("/", "--"),
            hash,
            name.split("/")[-1],
        ]
    )
    return module_path, hash


@dataclass
class ImportableModule:
    module_path: str
    hash: str


class _ScenarioModuleFactory:
    def get_module(self) -> ImportableModule:
        raise NotImplementedError


class LocalScenarioModuleFactory(_ScenarioModuleFactory):
    """Get the module of a local metric. The metric script is loaded
    from a local script."""

    def __init__(
        self,
        path: str,
        module_type: str = "scenarios",
        download_config: Optional[DownloadConfig] = None,
        download_mode: Optional[DownloadMode] = None,
        dynamic_modules_path: Optional[str] = None,
    ):
        """
        Example:
        x = "/home/text/mr.txt"
        Path(x).stem: mr
        Path(x).parent: /home/text
        """
        self.path = path
        self.module_type = module_type
        self.name = Path(path).stem
        self.download_config = download_config or DownloadConfig()
        self.download_mode = download_mode
        self.dynamic_modules_path = dynamic_modules_path

    def get_module(self) -> ImportableModule:
        # get script and other files
        imports = get_imports(self.path)
        local_imports = _download_additional_modules(
            name=self.name,
            base_path=str(Path(self.path).parent),
            imports=imports,
            download_config=self.download_config,
        )
        # copy the script and the files in an importable directory
        dynamic_modules_path = (
            self.dynamic_modules_path
            if self.dynamic_modules_path
            else init_dynamic_modules()
        )
        original_local_directory = os.path.dirname(self.path)
        # Copy additional files like dataset infos file if needed
        additional_files = [
            ("data", f"{original_local_directory}/data"),
            ("config.yaml", f"{original_local_directory}/config.yaml"),
        ]
        module_path, hash = _create_importable_file(
            local_path=self.path,
            local_imports=local_imports,
            additional_files=additional_files,
            dynamic_modules_path=dynamic_modules_path,
            module_namespace=self.module_type,
            name=self.name,
            download_mode=self.download_mode,  # type: ignore
        )
        # make the new module to be noticed by the import system
        importlib.invalidate_caches()
        return ImportableModule(module_path, hash)


class HubScenarioModuleFactory(_ScenarioModuleFactory):
    """Get the module of a metric from a metric repository on the Hub."""

    def __init__(
        self,
        name: str,
        module_type: str = "scenarios",
        revision: Optional[Union[str, Version]] = None,
        download_config: Optional[DownloadConfig] = None,
        download_mode: Optional[DownloadMode] = None,
        dynamic_modules_path: Optional[str] = None,
    ):
        self.name = name
        self.module_type = module_type
        self.revision = revision
        self.download_config = download_config or DownloadConfig()
        self.download_mode = download_mode
        self.dynamic_modules_path = dynamic_modules_path

    # def download_loading_script(self, revision) -> str:
    #     file_path = ep_hub_url(path=self.name,
    #                            name=self.name.split("/")[1] + ".py",
    #                            revision=revision)
    #     download_config = self.download_config.copy()
    #     if download_config.download_desc is None:
    #         download_config.download_desc = "Downloading builder script"
    #     return cached_path(file_path, download_config=download_config)

    def download_loading_script(self, revision: Optional[str]) -> str:
        file_path = ep_github_url(
            path=self.name, name=self.name + ".py", revision=revision
        )
        return cached_path(file_path, download_config=self.download_config)

    def get_module(self) -> ImportableModule:
        revision = self.revision or os.getenv(
            "EP_SCRIPTS_VERSION", config.SCRIPTS_VERSION
        )

        if re.match(
            r"\d*\.\d*\.\d*", revision  # type: ignore
        ):  # revision is version number (three digits separated by full stops)
            revision = (
                "v" + revision  # type: ignore
            )  # tagging convention on evaluate repository starts with v

        # get script and other files
        try:
            local_path = self.download_loading_script(revision)  # type: ignore
        except FileNotFoundError as err:
            # if there is no file found with current revision tag try to load main
            if (
                self.revision is None
                and os.getenv("ep_SCRIPTS_VERSION", config.SCRIPTS_VERSION) != "main"
            ):
                revision = "main"
                local_path = self.download_loading_script(revision)
            else:
                raise err

        imports = get_imports(local_path)
        local_imports = _download_additional_modules(
            name=self.name,
            base_path=ep_github_url(
                path=self.name, name="", revision=revision  # type: ignore
            ),  # type: ignore
            imports=imports,  # type: ignore
            download_config=self.download_config,
        )
        # copy the script and the files in an importable directory
        dynamic_modules_path = (
            self.dynamic_modules_path
            if self.dynamic_modules_path
            else init_dynamic_modules()
        )
        module_path, hash = _create_importable_file(
            local_path=local_path,
            local_imports=local_imports,
            additional_files=[],
            dynamic_modules_path=dynamic_modules_path,
            module_namespace=self.module_type,
            name=self.name,
            download_mode=self.download_mode,  # type: ignore
        )

        # make the new module to be noticed by the import system
        importlib.invalidate_caches()
        return ImportableModule(module_path, hash)


class CachedScenarioModuleFactory(_ScenarioModuleFactory):
    """
    Get the module of a metric that has been loaded once already and cached.
    The script that is loaded from the cache is the most recent one
     with a matching name.
    """

    def __init__(
        self,
        name: str,
        module_type: str = "scenarios",
        dynamic_modules_path: Optional[str] = None,
    ):
        self.name = name
        self.module_type = module_type
        self.dynamic_modules_path = dynamic_modules_path
        assert self.name.count("/") == 0

    def get_module(self) -> ImportableModule:
        dynamic_modules_path = (
            self.dynamic_modules_path
            if self.dynamic_modules_path
            else init_dynamic_modules()
        )
        importable_directory_path = os.path.join(
            dynamic_modules_path, self.module_type, self.name
        )

        hashes = (
            [h for h in os.listdir(importable_directory_path) if len(h) == 64]
            if os.path.isdir(importable_directory_path)
            else None
        )
        if not hashes:
            raise FileNotFoundError(
                f"Scenario {self.name} is not cached in {dynamic_modules_path}"
            )

        # get most recent

        def _get_modification_time(module_hash):
            return (
                (Path(importable_directory_path) / module_hash / (self.name + ".py"))
                .stat()
                .st_mtime
            )

        hash = sorted(hashes, key=_get_modification_time)[-1]
        logger.warning(
            f"Using the latest cached version of the module from"
            f" {os.path.join(importable_directory_path, hash)} "
            f"(last modified on {time.ctime(_get_modification_time(hash))}) since it "
            f"couldn't be found locally at {self.name}, or remotely on"
            f" the hub"
        )
        # make the new module to be noticed by the import system
        module_path = ".".join(
            [
                os.path.basename(dynamic_modules_path),
                self.module_type,
                self.name,
                hash,
                self.name,
            ]
        )
        importlib.invalidate_caches()
        return ImportableModule(module_path, hash)


def scenario_module_factory(
    path: str,  # type: ignore
    module_type: Optional[str] = None,
    revision: Optional[Union[str, Version]] = None,
    download_config: Optional[DownloadConfig] = None,
    download_mode: Optional[DownloadMode] = None,
    force_local_path: Optional[str] = None,
    dynamic_modules_path: Optional[str] = None,
    **download_kwargs,
) -> ImportableModule:
    """
    Download/extract/cache a metric module.
    Scenarios codes are cached inside the the dynamic modules cache to allow easy
     import (avoid ugly sys.path tweaks).
    Args:
        path (str): Path or name of the metric script.
            - if ``path`` is a local metric script or a directory containing
             a local metric script (if the script has the same name as the directory):
              -> load the module from the metric script
              e.g. ``'./metrics/accuracy'`` or ``'./metrics/accuracy/accuracy.py'``.
            - if ``path`` is a metric on the Hugging Face Hub (ex: `glue`, `squad`)
              -> load the module from the metric script in the github repository
               at expressai/datasets
              e.g. ``'accuracy'`` or ``'rouge'``.
        revision (Optional ``Union[str, datasets.Version]``):
            If specified, the module will be loaded from the datasets repository
             at this version.
            By default:
            - it is set to the local version of the lib.
            - it will also try to load it from the master branch if it's not
             available at the local version of the lib.
            Specifying a version that is different from your local version of the
             lib might cause compatibility issues.
        download_config (:class:`DownloadConfig`, optional): Specific download
         configuration parameters.
        download_mode (:class:`DownloadMode`, default ``REUSE_DATASET_IF_EXISTS``):
         Download/generate mode.
        force_local_path (Optional str): Optional path to a local path to download
         and prepare the script to.
            Used to inspect or modify the script folder.
        dynamic_modules_path (Optional str, defaults to EP_MODULES_CACHE /
         "datasets_modules", i.e. ~/.cache/expressai/modules/datasets_modules):
            Optional path to the directory in which the dynamic modules are saved.
             It must have been initialized with :obj:`init_dynamic_modules`.
            By default the datasets and metrics are stored inside the
             `datasets_modules` module.
        download_kwargs: optional attributes for DownloadConfig() which will
         override the attributes in download_config if supplied.
    Returns:
        ImportableModule
    """

    if download_config is None:
        download_config = DownloadConfig(**download_kwargs)
    download_mode = DownloadMode(download_mode or DownloadMode.REUSE_DATASET_IF_EXISTS)
    download_config.extract_compressed_file = True
    download_config.force_extract = True

    filename = list(filter(lambda x: x, path.replace(os.sep, "/").split("/")))[-1]
    if not filename.endswith(".py"):
        filename = filename + ".py"
    combined_path = os.path.join(path, filename)
    # Try locally
    if path.endswith(filename):
        if os.path.isfile(path):
            return LocalScenarioModuleFactory(
                path,
                download_config=download_config,
                download_mode=download_mode,
                dynamic_modules_path=dynamic_modules_path,
            ).get_module()
        else:
            raise FileNotFoundError(
                f"Couldn't find a metric script at {relative_to_absolute_path(path)}"
            )
    elif os.path.isfile(combined_path):
        return LocalScenarioModuleFactory(
            combined_path,
            download_config=download_config,
            download_mode=download_mode,
            dynamic_modules_path=dynamic_modules_path,
        ).get_module()
    elif is_relative_path(path) and path.count("/") <= 1 and not force_local_path:
        try:
            # load a canonical evaluation module from hub
            if path.count("/") == 0:
                # if no type provided look through all possible modules
                if module_type is None:
                    for current_type in [
                        "scenario",
                    ]:
                        try:
                            return HubScenarioModuleFactory(  # type: ignore
                                # f"evaluate-{current_type}/{path}",
                                path,
                                revision=revision,
                                download_config=download_config,
                                download_mode=download_mode,
                                dynamic_modules_path=dynamic_modules_path,
                            ).get_module()
                        except ConnectionError:
                            pass
                    raise FileNotFoundError
                # if module_type provided load specific module_type
                else:
                    return HubScenarioModuleFactory(  # type: ignore
                        # f"evaluate-{module_type}/{path}",
                        path,
                        revision=revision,
                        download_config=download_config,
                        download_mode=download_mode,
                        dynamic_modules_path=dynamic_modules_path,
                    ).get_module()
            # load community evaluation module from hub
            elif path.count("/") == 1:
                return HubScenarioModuleFactory(
                    path,
                    revision=revision,
                    download_config=download_config,
                    download_mode=download_mode,
                    dynamic_modules_path=dynamic_modules_path,
                ).get_module()
        except Exception as e1:  # noqa: all the attempts failed, before raising
            # the error we should check if the module is already cached.
            try:
                return CachedScenarioModuleFactory(
                    path, dynamic_modules_path=dynamic_modules_path
                ).get_module()
            except Exception as e2:  # noqa: if it's not in the cache, then
                # it doesn't exist.
                if not isinstance(e1, (ConnectionError, FileNotFoundError)):
                    raise e1 from None
                raise FileNotFoundError(
                    f"Couldn't find a module script at"
                    f" {relative_to_absolute_path(combined_path)}. "
                    f"Module '{path}' doesn't exist on the hub"
                ) from None
    else:
        raise FileNotFoundError(
            f"Couldn't find a module script at"
            f" {relative_to_absolute_path(combined_path)}."
        )
    return False  # type: ignore


def load(
    path: Optional[str] = None,
    config_name: Optional[str] = None,
    prompt: Optional[str] = None,
    formatter: Optional[str] = None,
    module_type: Optional[str] = None,
    process_id: int = 0,
    num_process: int = 1,
    cache_dir: Optional[str] = None,
    experiment_id: Optional[str] = None,
    keep_in_memory: bool = False,
    download_config: Optional[DownloadConfig] = None,
    download_mode: Optional[DownloadMode] = None,
    revision: Optional[Union[str, Version]] = None,
    **init_kwargs,
):
    """Load a `evaluate.EvaluationModule`.
    Args:
        path (``str``):
            path to the evaluation processing script with the evaluation builder
            . Can be either:
                - a local path to processing script or the directory containing
                 the script (if the script has the same name as the directory),
                    e.g. ``'./metrics/rouge'`` or ``'./metrics/rouge/rouge.py'``
                - a evaluation module identifier on the expressai evaluate
                 repo e.g. ``'rouge'`` or ``'bleu'`` that are in either ``'metrics/'``,
                    ``'comparisons/'``, or ``'measurements/'`` depending on
                     the provided ``module_type``.
        config_name (:obj:`str`, optional): selecting a configuration for the
         metric (e.g. the GLUE metric has a configuration for each subset)
        module_type (:obj:`str`, default ``'metric'``): type of evaluation module,
         can be one of ``'metric'``, ``'comparison'``, or ``'measurement'``.
        process_id (:obj:`int`, optional): for distributed evaluation: id of the process
        num_process (:obj:`int`, optional): for distributed evaluation: total
         number of processes
        cache_dir (Optional str): path to store the temporary predictions and
        references (default to `~/.cache/expressai/evaluate/`)
        experiment_id (``str``): A specific experiment id. This is used if
         several distributed evaluations share the same file system.
            This is useful to compute metrics in distributed setups (in particular
             non-additive metrics like F1).
        keep_in_memory (bool): Whether to store the temporary results in memory
         (defaults to False)
        download_config (Optional ``evaluate.DownloadConfig``: specific download
         configuration parameters.
        download_mode (:class:`DownloadMode`, default ``REUSE_DATASET_IF_EXISTS``):
         Download/generate mode.
        revision (Optional ``Union[str, evaluate.Version]``): if specified, the
         module will be loaded from the datasets repository
            at this version. By default it is set to the local version of the
             lib. Specifying a version that is different from
            your local version of the lib might cause compatibility issues.
    Returns:
        `evaluate.EvaluationModule`
    """

    if path is None:
        raise ValueError("a valid scenario name should be specified")

    download_mode = DownloadMode(download_mode or DownloadMode.REUSE_DATASET_IF_EXISTS)
    scenario_module = scenario_module_factory(
        path,
        module_type=module_type,
        revision=revision,
        download_config=download_config,
        download_mode=download_mode,
    )
    scenario_cls = import_main_class(scenario_module.module_path, ChatEval)
    library_dir = (
        config.DEFAULT_EP_MODULES_CACHE
        + "/"
        + "/".join(scenario_module.module_path.split(".")[0:-1])
    )
    scenario_instance = scenario_cls(config_directory=library_dir)

    # /usr2/home//pliu3/.cache/gair-nlp/modules/scenario_modules/scenarios/write_email/72f05ca5943b495c5f021e973e24149af5a2e4c1487f3361c39be27aa410c7d8/write_email/config.yaml
    # /usr2/home/pliu3/.cache/gair-nlp/modules/scenario_modules/scenarios/write_email/72f05ca5943b495c5f021e973e24149af5a2e4c1487f3361c39be27aa410c7d8
    return scenario_instance



import unittest

from chateval.metrics import get_metric


class MyTestCase(unittest.TestCase):

    dataset = [{"input": "write a movie review of Titanic"}]
    predictions = [
        'James Cameron\'s 1997 epic romantic disaster film "Titanic" tells the '
        "tragic story of two star-crossed lovers, Jack (Leonardo DiCaprio) and "
        "Rose (Kate Winslet), who fall in love aboard the ill-fated ship that met "
        "its infamous end in the North Atlantic on April 15, 1912. The film was a "
        "commercial and critical success, grossing over $2 billion worldwide "
        "and winning eleven Academy Awards, including Best Picture, Best Director, "
        'and Best Original Song. One of the most impressive aspects of "Titanic" '
        "is the film's stunning visual effects and production design. The "
        "detailed recreation of the Titanic and its sinking is both breathtaking "
        "and haunting, capturing the grandeur and tragedy of the ship's fate. The "
        "special effects used to bring the ship to life and simulate the sinking"
        " are still impressive more than two decades later. Another strong point "
        "of the film is the performances of the two leads, DiCaprio and Winslet. "
        "Their chemistry is palpable and their portrayal of two individuals from "
        "different social classes falling in love against all odds is touching and "
        "believable. The supporting cast, including Billy Zane and Gloria Stuart, "
        "also deliver strong performances that add depth to the film's characters"
        '. At its core, "Titanic" is a poignant love story set against the '
        "backdrop of a tragic historical event. The film expertly blends elements "
        "of romance, drama, and action to create an unforgettable cinematic "
        "experience. Despite its lengthy runtime of over three hours, the film is "
        "engaging and emotionally gripping throughout, leaving a lasting "
        'impression on viewers. Overall, "Titanic" is a cinematic masterpiece '
        "that stands the test of time. Cameron's epic film is a must-see for "
        "fans of romance, drama, and historical fiction, and remains a benchmark "
        "for blockbuster filmmaking."
    ]

    predictions_2 = [
        'James Cameron\'s 1997 epic romantic disaster film "Titanic" tells the '
    ]

    @unittest.skip("Skipping this test case for now due the need of OpenAI API key.")
    def test_GPTScore_helpfulness(self):

        metric = get_metric("generic_bool/relevance")
        result = metric.compute(self.dataset, self.predictions)
        print(result)

        metric = get_metric("generic_bool/coherence")
        result = metric.compute(self.dataset, self.predictions)
        print(result)

        self.assertEqual(result["value"], 1.0)
        self.assertEqual(result["sample_values"], [1.0])

    @unittest.skip("Skipping this test case for now due the need of OpenAI API key.")
    def test_GPTScore_general_likert(self):

        metric = get_metric("generic_likert/helpfulness")
        result = metric.compute(self.dataset, self.predictions)
        print(result)

        self.assertEqual(result["value"], 5.0)
        self.assertEqual(result["sample_values"], [5.0])

    @unittest.skip("Skipping this test case for now due the need of OpenAI API key.")
    def test_GPTScore_general_pairwise(self):

        metric = get_metric("generic_pairwise/helpfulness")
        result = metric.compare(self.dataset, self.predictions, self.predictions_2)
        print(result)

        self.assertEqual(result["value"], 1.0)
        self.assertEqual(result["sample_values"], [1.0])

    @unittest.skip("Skipping this test case for now due the need of OpenAI API key.")
    def test_GPTScore_general_rank(self):

        metric = get_metric("generic_rank/helpfulness")
        result = metric.rank(
            self.dataset, [[self.predictions[0], self.predictions_2[0]]]
        )
        print(result)

        self.assertEqual(result["value"], "1")
        self.assertEqual(result["sample_values"], ["1"])


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

import os
import unittest

from integration_tests.artifacts.utils import ARTIFACT_PATH

from chateval.chateval import ChatEval


class MyTestCase(unittest.TestCase):
    def test_load_metrics_from_config(self):
        artifact_path = os.path.join(ARTIFACT_PATH, "yaml_config")
        write_email_config_folder = os.path.join(artifact_path, "write_email")

        chateval = ChatEval(write_email_config_folder)

        self.assertEqual(len(chateval.metrics), 2)



# Copyright 2020 The HuggingFace Datasets Authors and the TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Utilities for file names."""

import re

_uppercase_uppercase_re = re.compile(r"([A-Z]+)([A-Z][a-z])")
_lowercase_uppercase_re = re.compile(r"([a-z\d])([A-Z])")

_single_underscore_re = re.compile(r"(?<!_)_(?!_)")
_multiple_underscores_re = re.compile(r"(_{2,})")

_split_re = r"^\w+(\.\w+)*$"


def camelcase_to_snakecase(name):
    """Convert camel-case string to snake-case."""
    name = _uppercase_uppercase_re.sub(r"\1_\2", name)
    name = _lowercase_uppercase_re.sub(r"\1_\2", name)
    return name.lower()



from __future__ import annotations

from hashlib import sha256
import logging
import re

import yaml

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

NO_ANSWER = "NO_ANSWER"


def get_yesno_from_text(text: str) -> dict:
    pattern = r"## (.+?)\n.+?([yn])"
    matches = re.findall(pattern, text, re.DOTALL)
    return {k: v for k, v in dict(matches).items()}


# example_text = """
# This is an example text with some ## important information about something.
# Please answer with either y or n: y
#
# ## Another piece of information that you should know about.
# Is this relevant? n
# """
#
# yesno_dict = get_yesno_from_text(example_text)
# print(yesno_dict)
# {'important information about something.': 'y',
# 'Another piece of information that you should know about.': 'n'}


def get_scores_from_text(text: str) -> dict:
    pattern = r"## (.+?)\n.+?(\d)/5"
    matches = re.findall(pattern, text, re.DOTALL)
    return {k: int(v) for k, v in dict(matches).items()}


def get_letter_from_data(data: str) -> str:
    last_y = (data.rfind("Y"), "Y")
    last_n = (data.rfind("N"), "N")

    if last_y[0] == -1 and last_n[0] == -1:
        return "YN"
    else:
        return max(last_y, last_n)[1]


def get_answer_from_data(data: str, answer_choices: list[str]) -> str:

    tuple_list = [(data.rfind(choice), choice) for choice in answer_choices]
    if sum([x[0] for x in tuple_list]) == -len(answer_choices):
        return NO_ANSWER
    else:
        return max(tuple_list)[1]


def dict_to_str(d: dict) -> str:
    return "\n" + "\n".join([f"{k}:{v}" for k, v in d.items()])


def list_to_str(d: list) -> str:
    return "\n" + "\n".join([f"{i+1}. {v}" for i, v in enumerate(d)])


def load_yaml(config_path: str) -> dict:
    with open(config_path, "r") as f:
        return yaml.safe_load(f)


def generate_format_func(pattern: str):
    """
    Generate a format function for a string with patter {}
    Args:
        pattern: a string with curly brackets, e.g., "[Query]:{query}\n[Documents]:{
        documents}\n[URL]:{url}"
    Returns:
    a function that takes an dictionary and returns a formatted string
    """

    arguments = re.findall(r"\{([^}]+)\}", pattern)

    if len(arguments) == 0:
        logger.warning(
            f"No arguments found in pattern: {pattern}, your input sample should be a "
            f"string, which will be regarded as the input"
        )
        return lambda d: d

    def format_func(input: dict[str, str]) -> str:
        """
        Generate a formatted string from the given dictionary.
        Args:
            input: a dictionary, for example
            input = {
                "query": "query",
                "documents": "documents",
                "url": "url"
            }
        Returns:
        a formatted string. for example: "[Query]:query\n[Documents]:documents\n[
        URL]:url"
        """

        result = pattern
        for arg in arguments:
            if arg not in input:
                raise KeyError(f"{arg} is not in your input sample")
            result = result.replace("{" + arg + "}", input[arg])

        return result

    return format_func


def hash_url_to_filename(url, etag=None):
    """
    Convert `url` into a hashed filename in a repeatable way.
    If `etag` is specified, append its hash to the url's, delimited
    by a period.
    If the url ends with .h5 (Keras HDF5 weights) adds '.h5' to the name
    so that TF 2.0 can identify it as a HDF5 file
    (see https://github.com/tensorflow/tensorflow/blob/00fad90125b18b80f
    e054de1055770cfb8fe4ba3/tensorflow/python/keras/engine/network.py#L1380)
    """
    url_bytes = url.encode("utf-8")
    url_hash = sha256(url_bytes)
    filename = url_hash.hexdigest()

    if etag:
        etag_bytes = etag.encode("utf-8")
        etag_hash = sha256(etag_bytes)
        filename += "." + etag_hash.hexdigest()

    if url.endswith(".py"):
        filename += ".py"

    return filename


def _hash_python_lines(lines: list[str]) -> str:
    filtered_lines = []
    for line in lines:
        line = re.sub(r"#.*", "", line)  # remove comments
        if line:
            filtered_lines.append(line)
    full_str = "\n".join(filtered_lines)

    # Make a hash from all this code
    full_bytes = full_str.encode("utf-8")
    return sha256(full_bytes).hexdigest()



import os
from pathlib import Path

# Version
SCRIPTS_VERSION = "main"
HUB_DEFAULT_VERSION = "main"

# Hub
EP_ENDPOINT = os.environ.get("EP_ENDPOINT", "http://gair-nlp.co")
HUB_SCENARIO_URL = (
    "https://raw.githubusercontent.com/gair-nlp/data/{revision}"
    "/scenarios/{path}/{name}"
)


# Cache location
DEFAULT_XDG_CACHE_HOME = "~/.cache"
XDG_CACHE_HOME = os.getenv("XDG_CACHE_HOME", DEFAULT_XDG_CACHE_HOME)
# "~/.cache/gair-nlp"
DEFAULT_EP_CACHE_HOME = os.path.join(XDG_CACHE_HOME, "gair-nlp")
EP_CACHE_HOME = os.path.expanduser(os.getenv("EP_HOME", DEFAULT_EP_CACHE_HOME))


# "~/.cache/gair-nlp/scenario"
DEFAULT_EP_SCENARIO_CACHE = os.path.join(EP_CACHE_HOME, "scenario")
EP_SCENARIO_CACHE = Path(os.getenv("EP_SCENARIO_CACHE", DEFAULT_EP_SCENARIO_CACHE))


# "~/.cache/gair-nlp/modules"
DEFAULT_EP_MODULES_CACHE = os.path.join(EP_CACHE_HOME, "modules")
EP_MODULES_CACHE = Path(os.getenv("EP_MODULES_CACHE", DEFAULT_EP_MODULES_CACHE))

# "~/.cache/gair-nlp/scenario/downloads"
DOWNLOADED_DATASETS_DIR = "downloads"
DEFAULT_DOWNLOADED_SCENARIO_PATH = os.path.join(
    EP_SCENARIO_CACHE, DOWNLOADED_DATASETS_DIR
)
DOWNLOADED_SCENARIO_PATH = Path(
    os.getenv("EP_DATASETS_DOWNLOADED_SCENARIO_PATH", DEFAULT_DOWNLOADED_SCENARIO_PATH)
)


# "~/.cache/gair-nlp/scenario/downloads/extracted"
EXTRACTED_SCENARIO_DIR = "extracted"
DEFAULT_EXTRACTED_SCENARIO_PATH = os.path.join(
    DEFAULT_DOWNLOADED_SCENARIO_PATH, EXTRACTED_SCENARIO_DIR
)
EXTRACTED_SCENARIO_PATH = Path(
    os.getenv("EP_DATASETS_EXTRACTED_SCENARIO_PATH", DEFAULT_EXTRACTED_SCENARIO_PATH)
)


# File names
LICENSE_FILENAME = "LICENSE"
MODULE_NAME_FOR_DYNAMIC_MODULES = "scenario_modules"
SCENARIO_INFO_FILENAME = "scenario_info.json"


# General environment variables accepted values for booleans
ENV_VARS_TRUE_VALUES = {"1", "ON", "YES", "TRUE"}
ENV_VARS_TRUE_AND_AUTO_VALUES = ENV_VARS_TRUE_VALUES.union({"AUTO"})

# Offline mode
EP_SCENARIO_OFFLINE = (
    os.environ.get("EP_SCENARIO_OFFLINE", "AUTO").upper() in ENV_VARS_TRUE_VALUES
)



from __future__ import annotations

from chateval.metrics.accuracy import Accuracy, AccuracyConfig
from chateval.metrics.count import Count, CountConfig
from chateval.metrics.gptscore import _GPT_METRICS
from chateval.metrics.metric import Metric
from chateval.metrics.rouge import Rouge, RougeConfig

_METRICS: [str, Metric] = {
    "rouge1": Rouge(RougeConfig(name="rouge", variety="rouge1")),
    "rouge2": Rouge(RougeConfig(name="rouge", variety="rouge2")),
    "rougeL": Rouge(RougeConfig(name="rouge", variety="rougeL")),
    "count": Count(CountConfig(name="count")),
    "accuracy": Accuracy(AccuracyConfig(name="accuracy")),
}

_METRICS.update(_GPT_METRICS)


def get_metric(name: str) -> Metric:
    return _METRICS[name]



from __future__ import annotations

import os
import pathlib
from typing import Final

# OPTIONAL_TEST_SUITES = ['cli_all']
OPTIONAL_TEST_SUITES: list[str] = []


ARTIFACT_PATH: Final = os.path.join(os.path.dirname(pathlib.Path(__file__)), "./")

top_path: Final = os.path.join(
    pathlib.Path(__file__).parent.parent.absolute(),
)



from __future__ import annotations

from dataclasses import dataclass

import numpy as np

from chateval.metrics.metric import Metric, MetricConfig


@dataclass
class CountConfig(MetricConfig):
    name = "count"

    def to_metric(self) -> Count:
        return Count(self)


class Count(Metric):
    def __init__(self, config: CountConfig):
        self.config = config

    def compute(self, dataset: list[dict], predictions: list[str]) -> dict:

        # TODO(pfliu-nlp): we should use the tokenizer to get the number of tokens
        sample_lengths = [len(hypo.split(" ")) for hypo in predictions]

        return {"value": np.mean(sample_lengths), "sample_values": sample_lengths}



import unittest

from chateval.kernels.openai import OpenAIChat, OpenAICompletion


class MyTestCase(unittest.TestCase):
    def test_completion(self):

        code = "Translate this sentence into Chinese: Hello, my name is Jack."
        result = OpenAICompletion().execute(code)
        print(result)

        result = OpenAIChat().execute(
            [
                {"role": "system", "content": "I will help with anything you need."},
                {"role": "user", "content": code},
            ]
        )
        print(result)
        self.assertIsNotNone(result)


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

from dataclasses import dataclass
import os
from typing import Optional

import openai

from chateval.kernels.kernel import Kernel, KernelConfig

openai.api_key = os.environ.get("OPENAI_API_KEY")


@dataclass
class OpenAICompletionConfig(KernelConfig):

    model_name: str = "text-curie-001"
    max_tokens: int = 64
    temperature: float = 0.7
    top_p: float = 1
    suffix: str = ""
    frequency_penalty: float = 0.0
    presence_penalty: float = 0.0
    logprobs: float = 1
    n: int = 1
    echo: bool = True
    stop: object = None


class OpenAICompletion(Kernel):
    def __init__(self, config: Optional[OpenAICompletionConfig] = None):
        self.config = config if config is not None else OpenAICompletionConfig()

    def execute(self, code) -> dict[str, str]:

        response = openai.Completion.create(
            model=self.config.model_name,
            prompt=code,
            max_tokens=self.config.max_tokens,
            temperature=self.config.temperature,
            top_p=self.config.top_p,
            logprobs=self.config.logprobs,
            frequency_penalty=self.config.frequency_penalty,
            presence_penalty=self.config.presence_penalty,
            n=self.config.n,
            echo=self.config.echo,
            stop=self.config.stop,
        )

        return response["choices"][0]["text"]


@dataclass
class OpenAIChatConfig(KernelConfig):
    model_name: str = "gpt-3.5-turbo"
    max_tokens: int = 256
    temperature: float = 0.0
    top_p: float = 1
    frequency_penalty: float = 0.0
    presence_penalty: float = 0.0
    n: int = 1


class OpenAIChat(Kernel):
    def __init__(self, config: Optional[OpenAIChatConfig] = None):
        self.config = config if config is not None else OpenAIChatConfig()

    def execute(self, code) -> dict[str, str]:
        response = openai.ChatCompletion.create(
            model=self.config.model_name,
            messages=code,
            max_tokens=self.config.max_tokens,
            temperature=self.config.temperature,
            top_p=self.config.top_p,
            frequency_penalty=self.config.frequency_penalty,
            presence_penalty=self.config.presence_penalty,
            n=self.config.n,
        )

        output = response["choices"][0]["message"]["content"]

        return output



import pathlib
import unittest

from chateval import load

current_path = pathlib.Path(__file__).resolve().parent


class MyTestCase(unittest.TestCase):
    def test_scenario(self):

        scenario = load(f"{current_path}")
        predictions = [
            "My name is [name], and I am currently a student in your [class name]",
        ]

        result = scenario.evaluate(predictions)

        self.assertAlmostEqual(result["rouge1"]["value"], 0.21374, 3)
        self.assertAlmostEqual(result["count"]["value"], 14.0, 3)



from __future__ import annotations

import dataclasses
from dataclasses import dataclass
import os
from typing import Optional

import openai

from chateval.kernels.kernel import Kernel, KernelConfig

openai.api_key = os.environ["OPENAI_API_KEY"]


@dataclass
class OpenAICompletionConfig(KernelConfig):
    model_name: str = "text-curie-001"
    max_tokens: int = 64
    temperature: float = 0.7
    top_p: float = 1
    suffix: str = ""
    frequency_penalty: float = 0.0
    presence_penalty: float = 0.0
    logprobs: float = 1
    n: int = 1
    echo: bool = False
    stop: object = None

    def to_kernel(self) -> Kernel:
        return OpenAICompletion(self)

    @classmethod
    def from_dict(cls, data_dict: dict) -> OpenAICompletionConfig:
        field_names = set(f.name for f in dataclasses.fields(cls))
        return cls(**{k: v for k, v in data_dict.items() if k in field_names})


class OpenAICompletion(Kernel):
    def __init__(self, config: Optional[OpenAICompletionConfig] = None):

        self.config = (
            config
            if config is not None
            else OpenAICompletionConfig(
                model_name="text-curie-001",
                suffix="",
                temperature=0,
                max_tokens=10,
                logprobs=1,
                top_p=1,
                frequency_penalty=0,
                presence_penalty=0,
                n=1,
                echo=False,
                stop=None,
            )
        )

    def execute(self, code) -> dict[str, str]:
        # Set the API key from the environment variable

        response = openai.Completion.create(
            model=self.config.model_name,
            prompt=code,
            max_tokens=self.config.max_tokens,
            temperature=self.config.temperature,
            top_p=self.config.top_p,
            frequency_penalty=self.config.frequency_penalty,
            presence_penalty=self.config.presence_penalty,
            logprobs=self.config.logprobs,
            n=self.config.n,
            echo=self.config.echo,
            stop=self.config.stop,
        )

        out = {
            "text": response["choices"][0]["text"],
            "token_logprobs": response["choices"][0]["logprobs"]["token_logprobs"],
            "tokens": response["choices"][0]["logprobs"]["tokens"],
        }

        return out



from __future__ import annotations

import abc
from dataclasses import dataclass


@dataclass
class MetricConfig:
    name: str

    @abc.abstractmethod
    def to_metric(self) -> Metric:
        return Metric(self)


class Metric:
    def compute(self, predictions: list[str], dataset: list[dict]) -> dict:
        raise NotImplementedError()



from __future__ import annotations

from dataclasses import dataclass
import logging
import os
from typing import Optional

from tqdm import tqdm

from chateval.kernels.openai import OpenAIChat, OpenAIChatConfig
from chateval.metrics.metric import Metric, MetricConfig
from chateval.metrics.protocols.utils import PROTOCOLS_PATH
from chateval.utils.prompt_utils import format_necessary
from chateval.utils.py_util import (
    dict_to_str,
    get_answer_from_data,
    list_to_str,
    load_yaml,
    NO_ANSWER,
)

logging.basicConfig(level=logging.INFO)


@dataclass
class GPTScoreConfig(MetricConfig):
    name: str
    protocol_config_path: str
    criteria: Optional[str] = None
    eval_model_config: OpenAIChatConfig = OpenAIChatConfig(
        model_name="gpt-3.5-turbo",
    )

    def __post_init__(self):
        self.protocol_config = load_yaml(self.protocol_config_path)


class GPTScore(Metric):
    def __init__(self, config: GPTScoreConfig):
        self.config = config
        self.evaluator = OpenAIChat(config.eval_model_config)

    def compute_sample(self, sample: dict, prediction: str) -> dict:

        prompt = self.config.protocol_config["prompt"]
        prompt_args = {}
        # instantiate prompt with criteria
        if self.config.criteria is not None:
            if self.config.criteria not in self.config.protocol_config["criteria"]:
                raise ValueError(
                    f"Criteria {self.config.criteria} not found in protocol config."
                )

            criteria_content = self.config.protocol_config["criteria"][
                self.config.criteria
            ]
            # likert protocol
            if self.config.protocol_config["eval_type"] == "cot_likert":
                criteria_content = dict_to_str(criteria_content)
                prompt_args["choices"] = ", ".join(
                    list(self.config.protocol_config["choice_scores"].keys())
                )

            prompt_args["criteria"] = criteria_content

        # instantiate prompt with input and completion
        # TODO(pfliu-nlp): the for loop is problematic
        for input_slot_name, output_slot_name in self.config.protocol_config[
            "input_outputs"
        ].items():
            prompt_args[input_slot_name] = sample[input_slot_name]
            prompt_args[output_slot_name] = prediction

        prompt_instantiated = format_necessary(prompt, **prompt_args)
        result = self.evaluator.execute(
            [
                {"role": "user", "content": prompt_instantiated},
            ]
        )

        extracted_result = get_answer_from_data(
            result, list(self.config.protocol_config["choice_scores"].keys())
        )

        if extracted_result in self.config.protocol_config["choice_scores"]:
            return {
                "value": self.config.protocol_config["choice_scores"][extracted_result],
                "detail": {
                    "prompt": prompt_instantiated,
                    "judgment": result,
                },
            }
        else:
            return {
                "value": -1,
                "detail": {
                    "prompt": prompt_instantiated,
                    "judgment": result,
                },
            }

    def compute(self, dataset: list[dict], predictions: list[str]) -> dict:
        results = []
        for sample, prediction in zip(tqdm(dataset), predictions):
            try:
                out = self.compute_sample(sample, prediction)
            except (ValueError, TypeError, RuntimeError) as e:
                logging.warning(f"Error in computing GPTScore: {e}")
                out = None
            results.append(out)
        scores = [result["value"] if result is not None else None for result in results]
        details = [
            result["detail"] if result is not None else None for result in results
        ]

        value = 0
        no_score = 0
        for score in scores:
            if score is not None and score >= 0:
                value += score
            else:
                no_score += 1
        value = value / (len(scores) - no_score)
        return {
            "value": value,
            "no_score": no_score,
            "sample_values": scores,
            "details": details,
        }

    def compare(
        self, dataset: list[dict], predictions_1: list[str], predictions_2: list[str]
    ) -> dict:
        results = []
        for sample, prediction_1, predictions_2 in zip(
            tqdm(dataset), predictions_1, predictions_2
        ):
            try:
                out = self.compare_sample(sample, prediction_1, predictions_2)
            except (ValueError, TypeError, RuntimeError) as e:
                logging.warning(f"Error in computing GPTScore: {e}")
                out = None
            results.append(out)
        scores = [result["value"] if result is not None else None for result in results]
        details = [
            result["detail"] if result is not None else None for result in results
        ]

        value = 0
        no_score = 0
        for score in scores:
            if score is not None and score >= 0:
                value += score
            else:
                no_score += 1
        value = value / (len(scores) - no_score)
        return {
            "value": value,
            "no_score": no_score,
            "sample_values": scores,
            "details": details,
        }

    def compare_sample(
        self, sample: dict, prediction_1: str, prediction_2: str
    ) -> dict:

        prompt = self.config.protocol_config["prompt"]
        prompt_args = {}
        # instantiate prompt with criteria
        if self.config.criteria is not None:
            if self.config.criteria not in self.config.protocol_config["criteria"]:
                raise ValueError(
                    f"Criteria {self.config.criteria} not found in protocol config."
                )

            criteria_content = self.config.protocol_config["criteria"][
                self.config.criteria
            ]
            prompt_args["criteria"] = criteria_content

        # instantiate prompt with input and completion
        # TODO(pfliu-nlp): the for loop is problematic

        for input_slot_name, output_slot_names in self.config.protocol_config[
            "input_outputs"
        ].items():
            prompt_args[input_slot_name] = sample[input_slot_name]
            prompt_args[output_slot_names[0]] = prediction_1
            prompt_args[output_slot_names[1]] = prediction_2

        prompt_instantiated = format_necessary(prompt, **prompt_args)
        result = self.evaluator.execute(
            [
                {"role": "user", "content": prompt_instantiated},
            ]
        )

        extracted_result = get_answer_from_data(
            result, list(self.config.protocol_config["choice_scores"].keys())
        )

        if extracted_result in self.config.protocol_config["choice_scores"]:
            return {
                "value": self.config.protocol_config["choice_scores"][extracted_result],
                "detail": {
                    "prompt": prompt_instantiated,
                    "judgment": result,
                },
            }
        else:
            return {
                "value": -1,
                "detail": {
                    "prompt": prompt_instantiated,
                    "judgment": result,
                },
            }

    def rank(
        self,
        dataset: list[dict],
        predictions_list: list[list[str]],
    ) -> dict:
        results = []
        print("predictions_list", predictions_list)
        for sample, predictions in zip(tqdm(dataset), predictions_list):
            try:
                out = self.rank_sample(sample, predictions)
            except (ValueError, TypeError, RuntimeError) as e:
                logging.warning(f"Error in computing GPTScore: {e}")
                out = None
            results.append(out)
        scores = [result["value"] if result is not None else None for result in results]
        details = [
            result["detail"] if result is not None else None for result in results
        ]

        dict_sys_choice = {}
        no_score = 0
        print(scores)
        for score in scores:
            if score is not None and score != NO_ANSWER:
                if score not in dict_sys_choice:
                    dict_sys_choice[score] = 1
                else:
                    dict_sys_choice[score] += 1
            else:
                no_score += 1

        # get the most frequent choice
        value = max(dict_sys_choice, key=dict_sys_choice.get)

        return {
            "value": value,
            "no_score": no_score,
            "sample_values": scores,
            "details": details,
        }

    def rank_sample(self, sample: dict, predictions: list[str]) -> dict:

        prompt = self.config.protocol_config["prompt"]
        prompt_args = {}
        # instantiate prompt with criteria
        if self.config.criteria is not None:
            if self.config.criteria not in self.config.protocol_config["criteria"]:
                raise ValueError(
                    f"Criteria {self.config.criteria} not found in protocol config."
                )

            criteria_content = self.config.protocol_config["criteria"][
                self.config.criteria
            ]
            prompt_args["criteria"] = criteria_content

        for input_slot_name, output_slot_name in self.config.protocol_config[
            "input_outputs"
        ].items():
            prompt_args[input_slot_name] = sample[input_slot_name]
            prompt_args[output_slot_name] = list_to_str(predictions)

        prompt_args["n"] = len(predictions)

        prompt_instantiated = format_necessary(prompt, **prompt_args)
        result = self.evaluator.execute(
            [
                {"role": "user", "content": prompt_instantiated},
            ]
        )

        choices = [str(i + 1) for i in range(len(predictions))]
        extracted_result = get_answer_from_data(result, choices)

        if extracted_result in choices:
            return {
                "value": extracted_result,
                "detail": {
                    "prompt": prompt_instantiated,
                    "judgment": result,
                },
            }
        else:
            return {
                "value": NO_ANSWER,
                "detail": {
                    "prompt": prompt_instantiated,
                    "judgment": result,
                },
            }


_GPT_METRICS: [str, Metric] = {
    "generic_bool/relevance": GPTScore(
        GPTScoreConfig(
            name="generic_bool/relevance",
            protocol_config_path=os.path.join(PROTOCOLS_PATH, "generic_bool.yaml"),
            criteria="relevance",
        ),
    ),
    "generic_bool/coherence": GPTScore(
        GPTScoreConfig(
            name="generic_bool/coherence",
            protocol_config_path=os.path.join(PROTOCOLS_PATH, "generic_bool.yaml"),
            criteria="coherence",
        ),
    ),
    "generic_bool/helpfulness": GPTScore(
        GPTScoreConfig(
            name="generic_bool/helpfulness",
            protocol_config_path=os.path.join(PROTOCOLS_PATH, "generic_bool.yaml"),
            criteria="helpfulness",
        ),
    ),
    "generic_bool/grammar": GPTScore(
        GPTScoreConfig(
            name="generic_bool/grammar",
            protocol_config_path=os.path.join(PROTOCOLS_PATH, "generic_bool.yaml"),
            criteria="grammar",
        ),
    ),
    "generic_likert/helpfulness": GPTScore(
        GPTScoreConfig(
            name="generic_likert/helpfulness",
            protocol_config_path=os.path.join(PROTOCOLS_PATH, "generic_likert.yaml"),
            criteria="helpfulness",
        ),
    ),
    "generic_likert/relevance": GPTScore(
        GPTScoreConfig(
            name="generic_likert/relevance",
            protocol_config_path=os.path.join(PROTOCOLS_PATH, "generic_likert.yaml"),
            criteria="relevance",
        ),
    ),
    "generic_likert/coherence": GPTScore(
        GPTScoreConfig(
            name="generic_likert/coherence",
            protocol_config_path=os.path.join(PROTOCOLS_PATH, "generic_likert.yaml"),
            criteria="coherence",
        ),
    ),
    "generic_pairwise/helpfulness": GPTScore(
        GPTScoreConfig(
            name="generic_pairwise/coherence",
            protocol_config_path=os.path.join(PROTOCOLS_PATH, "generic_pairwise.yaml"),
            criteria="helpfulness",
        ),
    ),
    "generic_pairwise/relevance": GPTScore(
        GPTScoreConfig(
            name="generic_pairwise/relevance",
            protocol_config_path=os.path.join(PROTOCOLS_PATH, "generic_pairwise.yaml"),
            criteria="relevance",
        ),
    ),
    "generic_pairwise/coherence": GPTScore(
        GPTScoreConfig(
            name="generic_pairwise/coherence",
            protocol_config_path=os.path.join(PROTOCOLS_PATH, "generic_pairwise.yaml"),
            criteria="coherence",
        ),
    ),
    "generic_rank/helpfulness": GPTScore(
        GPTScoreConfig(
            name="generic_rank/helpfulness",
            protocol_config_path=os.path.join(PROTOCOLS_PATH, "generic_rank.yaml"),
            criteria="helpfulness",
        ),
    ),
    "generic_rank/relevance": GPTScore(
        GPTScoreConfig(
            name="generic_rank/relevance",
            protocol_config_path=os.path.join(PROTOCOLS_PATH, "generic_rank.yaml"),
            criteria="relevance",
        ),
    ),
    "generic_rank/coherence": GPTScore(
        GPTScoreConfig(
            name="generic_rank/coherence",
            protocol_config_path=os.path.join(PROTOCOLS_PATH, "generic_rank.yaml"),
            criteria="coherence",
        ),
    ),
}



from __future__ import annotations

import json

import yaml

from chateval.constants.models import ModelType
from chateval.metrics import get_metric


class ChatEval:
    def __init__(self, config_directory: str = None):
        self.config_directory = config_directory
        self.config = self._config()
        self.dataset = self._dataset()
        self.metrics = self._metrics()

    def _config_name(self):
        raise NotImplementedError()

    def _config(self):
        return self.load_config(f"{self.config_directory}/config.yaml")

    def _dataset(self):

        settings = self.config["settings"]
        default_setting = self.config["default_setting"]
        if default_setting not in settings:
            raise ValueError("The default setting is not in the settings")
        data_path = settings[default_setting]["dataset"]

        return self.load_dataset(f"{self.config_directory}/{data_path}")

    def _metrics(self):

        metrics = {
            metric_config["name"]: get_metric(metric_config["name"])
            for metric_config in self.config["metrics"]
        }

        return metrics

    def get_default_setting_config(self):
        return self.config["settings"][self.config["default_setting"]]

    def inference_one_sample(self, sample: dict) -> str:
        raise NotImplementedError()

    def inference_batch_samples(self, samples: list[dict]) -> list[str]:
        raise NotImplementedError()

    def evaluate(
        self, model: str | list[str], model_type: ModelType = ModelType.output
    ) -> dict:

        if model_type == ModelType.metric:
            return {
                name: metric.compute(self.dataset, metric_model=model)
                for name, metric in self.metrics.items()
            }
        elif model_type == ModelType.output:
            return {
                name: metric.compute(self.dataset, predictions=model)
                for name, metric in self.metrics.items()
            }
        else:
            raise ValueError(
                "The predictions should be either a list of string or a string"
            )

    def load_config(self, path: str) -> dict:
        # judge if the file is yaml, raise error if not
        if not path.endswith(".yaml") and not path.endswith(".yml"):
            raise ValueError("The config file should be yaml file")

        with open(path, "r") as yaml_file:
            yaml_data = yaml.load(yaml_file, Loader=yaml.FullLoader)
        return yaml_data

    def load_dataset(self, path: str) -> list[dict]:
        """
        Load the dataset from the path
        Args:
            path: it should be a jsonl file

        Returns:
            a list of dict, each dict is a sample
        """

        # judge if the file is jsonl, raise error if not
        if not path.endswith(".jsonl"):
            raise ValueError("The dataset file should be jsonl file")

        with open(path, "r") as f:
            dataset = [json.loads(line) for line in f]
        return dataset



from __future__ import annotations

import abc
import dataclasses
from dataclasses import dataclass
from typing import Any


@dataclass
class KernelConfig:
    @abc.abstractmethod
    def to_kernel(self) -> Kernel:
        return Kernel(self)

    @classmethod
    def from_dict(cls, data_dict: dict) -> Kernel:
        field_names = set(f.name for f in dataclasses.fields(cls))
        return cls(**{k: v for k, v in data_dict.items() if k in field_names})


class Kernel:
    def __init__(self, config: Any):
        self.config = config



import enum

from chateval.utils.logging import get_logger

logger = get_logger(__name__)


class DownloadMode(enum.Enum):

    REUSE_DATASET_IF_EXISTS = "reuse_dataset_if_exists"
    REUSE_CACHE_IF_EXISTS = "reuse_cache_if_exists"
    FORCE_REDOWNLOAD = "force_redownload"



"""Definition of different types of models."""
from __future__ import annotations

from enum import Enum


class ModelType(str, Enum):
    """Model types available in this tool."""

    api = "api"
    metric = "metric"
    output = "output"

    @staticmethod
    def list() -> list[str]:
        """Obtains string representations of all values.
        Returns:
            List of all values in str.
        """
        return list(map(lambda c: c.value, ModelType))



# Copyright 2020 The HuggingFace Datasets Authors and the TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Version utils."""

import dataclasses
from dataclasses import dataclass
import re
from typing import Optional, Union

_VERSION_TMPL = r"^(?P<major>{v})" r"\.(?P<minor>{v})" r"\.(?P<patch>{v})$"
_VERSION_WILDCARD_REG = re.compile(_VERSION_TMPL.format(v=r"\d+|\*"))
_VERSION_RESOLVED_REG = re.compile(_VERSION_TMPL.format(v=r"\d+"))


@dataclass
class Version:
    """Dataset version MAJOR.MINOR.PATCH.
    Args:
        version_str (:obj:`str`): Eg: "1.2.3".
        description (:obj:`str`): A description of what is new in this version.
    Attributes:
        version_str (:obj:`str`): Eg: "1.2.3".
        description (:obj:`str`): A description of what is new in this version.
        major (:obj:`str`):
        minor (:obj:`str`):
        patch (:obj:`str`):
    Example:
    ```py
    >>> VERSION = datasets.Version("1.0.0")
    ```
    """

    version_str: str
    description: Optional[str] = None
    major: Optional[Union[str, int]] = None
    minor: Optional[Union[str, int]] = None
    patch: Optional[Union[str, int]] = None

    def __post_init__(self):
        self.major, self.minor, self.patch = _str_to_version(self.version_str)

    def __repr__(self):
        return f"{self.tuple[0]}.{self.tuple[1]}.{self.tuple[2]}"

    @property
    def tuple(self):
        return self.major, self.minor, self.patch

    def _validate_operand(self, other):
        if isinstance(other, str):
            return Version(other)
        elif isinstance(other, Version):
            return other
        raise TypeError(f"{other} (type {type(other)}) cannot be compared to version.")

    def __eq__(self, other):
        try:
            other = self._validate_operand(other)
        except (TypeError, ValueError):
            return False
        else:
            return self.tuple == other.tuple

    def __ne__(self, other):
        return not self.__eq__(other)

    def __lt__(self, other):
        other = self._validate_operand(other)
        return self.tuple < other.tuple

    def __le__(self, other):
        other = self._validate_operand(other)
        return self.tuple <= other.tuple

    def __gt__(self, other):
        other = self._validate_operand(other)
        return self.tuple > other.tuple

    def __ge__(self, other):
        other = self._validate_operand(other)
        return self.tuple >= other.tuple

    def match(self, other_version):
        """Returns True if other_version matches.
        Args:
            other_version: string, of the form "x[.y[.x]]" where {x,y,z} can be a
                number or a wildcard.
        """
        major, minor, patch = _str_to_version(other_version, allow_wildcard=True)
        return (
            major in [self.major, "*"]
            and minor in [self.minor, "*"]
            and patch in [self.patch, "*"]
        )

    @classmethod
    def from_dict(cls, dic):
        field_names = {f.name for f in dataclasses.fields(cls)}
        return cls(**{k: v for k, v in dic.items() if k in field_names})

    def _to_yaml_string(self) -> str:
        return self.version_str


def _str_to_version(version_str, allow_wildcard=False):
    """Return the tuple (major, minor, patch) version extracted from the str."""
    reg = _VERSION_WILDCARD_REG if allow_wildcard else _VERSION_RESOLVED_REG
    res = reg.match(version_str)
    if not res:
        msg = f"Invalid version '{version_str}'. Format should be x.y.z"
        if allow_wildcard:
            msg += " with {x,y,z} being digits or wildcard."
        else:
            msg += " with {x,y,z} being digits."
        raise ValueError(msg)
    return tuple(
        v if v == "*" else int(v)
        for v in [res.group("major"), res.group("minor"), res.group("patch")]
    )



from __future__ import annotations

from dataclasses import dataclass
from typing import Optional

from chateval.metrics.gptscore import _GPT_METRICS
from chateval.metrics.metric import Metric, MetricConfig


@dataclass
class AccuracyConfig(MetricConfig):
    name = "accuracy"
    meta_eval: bool = False

    def to_metric(self) -> Accuracy:
        return Accuracy(self)


class Accuracy(Metric):
    def __init__(self, config: AccuracyConfig):
        self.config = config

    def compute(
        self,
        dataset: list[dict],
        predictions: Optional[list[str]] = None,
        metric_model: Optional[str] = None,
    ) -> dict:

        if predictions is None and metric_model is None:
            raise ValueError("Either predictions or metric_model must be provided.")

        if metric_model is not None:
            prediction_infos = _GPT_METRICS[metric_model].compute(
                dataset, [x["completion"] for x in dataset]
            )
            predictions = [x for x in prediction_infos["sample_values"]]

        if len(dataset) != len(predictions):
            raise ValueError("The number of samples and predictions do not match.")
        if len(dataset) == 0:
            raise ValueError("No dataset to compute.")

        values = []

        for sample, pred in zip(dataset, predictions):
            # TODO: only consider the first reference
            values.append(int(sample["references"][0] == pred))

        return {"value": sum(values) / len(values), "sample_values": values}
